# -*- coding: utf-8 -*-
#!/usr/bin/env python

# Copyright (C) 2008 Eben Eliason
# Copyright (C) 2013 Jorge Alberto Gómez López

#  This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import xml.dom.minidom
import getopt
import re
import os
import string

HELP = '''\nUsage: sugar-iconify.py [options] input.svg\n
Options:\n
 -c\t\tApply default color entities (#666666, #ffffff) to output
 -d directory\tThe preferred output directory
 -e\t\tDo not insert entities for strokes and fills
 -f hex\t\tHex value to replace with fill entity
 -g\t\tAutomatically accept guesses for stroke and fill entities
 -h\t\tDisplay this help message
 -i\t\tInsert "isolated stroke" entities
 -m\t\tMultiple export; export top level groups as separate icons
 -o\t\tOverwrite the input file; overridden by -m
 -p pattern\tOnly export icons whose name matches pattern; for use with -m
 -s hex\t\tHex value to replace with stroke entity
 -x\t\tOutput example SVGs, for previewing their appearance in Sugar;
   \t\tignored with -m
 -v\t\tverbose'''


class SugarIconify():

    def __init__(self, command_line=False):
        # Declare variables and constants
        self.default_stroke_color = '#666666'
        self.default_fill_color = '#ffffff'
        self.transparent_color = '#00000000'
        self.stroke_color = self.default_stroke_color
        self.fill_color = self.default_fill_color
        self._target_stroke = None
        self._target_fill = None
        self.stroke_entity = 'stroke_color'
        self.fill_entity = 'fill_color'
        self.iso_stroke_entity = 'iso_stroke_color'

        self.output_path = ''
        self.pattern = ''
        self.entities_passed = 0
        self.use_default_colors = False
        self.confirm_guess = True
        self.use_entities = True
        self.multiple = False
        self.verbose = False
        self.overwrite_input = False
        self.output_examples = False
        self.use_iso_strokes = False

        if command_line:
            self._parse_command_line()

    def usage(self):
        # Define help output
        print HELP

    def _parse_command_line(self):
        ''' Try to make sense of the command-line arguments. '''
        try:
            opts, arg = getopt.getopt(sys.argv[1:], 's:f:gcd:imp:oehvx',
                                      ['stroke=', 'fill=', 'guess', 'help',
                                       'overwrite', 'verbose'])
        except BaseException:
            self.usage()
            sys.exit(2)

        if len(arg) < 1:
            self.usage()
            sys.exit(2)

        # Interpret arguments
        for o, a in opts:
            if o in ['-s', '--stroke']:
                self.set_stroke_color(a)
            elif o in ['-f', '--fill']:
                self.set_fill_color(a)
            elif o in ['-g', '--guess']:
                self.set_confirm_guess(False)
            elif o == '-c':
                self.set_use_default_colors(True)
            elif o in ['-o', '--overwrite']:
                self.set_overwrite_input(True)
            elif o == '-d':
                self.set_output_path(a)
            elif o == '-e':
                self.set_use_entities(False)
            elif o in ['-v', 'verbose']:
                self.set_verbose(True)
            elif o == '-p':
                self.set_pattern(a)
            elif o in ['-h', '--help']:
                usage()
                sys.exit(2)
            elif o == '-m':
                self.set_multiple(True)
            elif o == '-x':
                self.set_output_examples(True)
            elif o == '-i':
                self.set_use_iso_strokes(True)

        self.iconify(arg[0])

    def rgb_to_hex(self, rgb_str):
        s = re.sub(r'.*rgb\(([^)]*).*', r'\1', rgb_str)
        percent_list = s.split(',')
        hex_str = '#'
        for value in percent_list:
            hex_str += self.percent_to_hex(value)
        return hex_str

    def percent_to_hex(self, num):
        number = float(num.strip()[:-1])
        decimal = (number * 255) / 100
        decimal = int(round(decimal, 0))
        hex_val = hex(decimal).split('x')[1]
        if len(hex_val) == 1:
            hex_val = '0' + hex_val
        return hex_val

    def set_stroke_color(self, s=None):
        if s is not None:
            if 'rgb' in s.lower():
                self.stroke_color = self.rgb_to_hex(s)
                self._target_stroke = s
            else:
                self.stroke_color = '#' + s.lstrip('#').lower()
            self.entities_passed += 1

    def set_fill_color(self, f=None):
        if f is not None:
            if 'rgb' in f.lower():
                self.fill_color = self.rgb_to_hex(f)
                self._target_fill = f
            else:
                self.fill_color = '#' + f.lstrip('#').lower()
            self.entities_passed += 1

    def set_confirm_guess(self, g=False):
        self.confirm_guess = g

    def set_use_default_colors(self, c=False):
        self.use_default_colors = c

    def set_overwrite_input(self, o=False):
        self.overwrite_input = o

    def set_output_path(self, d=None):
        if d is not None:
            self.output_path = d.rstrip('/') + '/'

    def set_use_entity(self, e=False):
        self.use_entities = e

    def set_verbose(self, v=False):
        self.verbose = v

    def set_pattern(self, p=None):
        if p is not None:
            self.pattern = p

    def set_multiple(self, m=False):
        self.multiple = m

    def set_output_examples(self, x=False):
        self.output_examples = x

    def set_use_iso_strokes(self, i=False):
        self.use_iso_strokes = i

    def iconify(self, file_path):
        # Isolate important parts of the input path
        self.svgfilepath = file_path
        self.svgdirpath, self.sep, self.svgfilename = \
            self.svgfilepath.rpartition('/')
        svgbasename = re.sub(r'(.*)\.([^.]+)', r'\1', self.svgfilename)

        # Load the SVG as text
        try:
            self.svgfile = open(self.svgfilepath, 'r')
        except BaseException:
            sys.exit('Error: Could not locate ' + self.svgfilepath)

        try:
            self.svgtext = self.svgfile.read()
            self.svgfile.close()
        except BaseException:
            self.svgfile.close()
            sys.exit('Error: Could not read ' + self.svgfilepath)

        # Determine the creator of the SVG (we only care about
        # Inkscape and Illustrator)
        self.creator = 'unknown'

        if re.search('illustrator', self.svgtext, re.I):
            self.creator = 'illustrator'
        elif re.search('inkscape', self.svgtext, re.I):
            self.creator = 'inkscape'

        if self.verbose:
            print 'The self.creator of this svg is ' + self.creator + '.'

        # Hack the entities into the readonly DTD
        if self.use_entities:

            # Before replacing them, we read the stroke/fill values
            # out, should they have previously been defined, to prevent
            # needing to make guesses for them later
            self.stroke_match = re.search(r'stroke_color\s*\"([^"]*)\"',
                                          self.svgtext)
            self.fill_match = re.search(r'fill_color\s*\"([^"]*)\"',
                                        self.svgtext)

            if self.stroke_match is not None:
                self.stroke_color = self.stroke_match.group(1).lower()
                self.entities_passed += 1
            if self.fill_match is not None:
                self.fill_color = self.fill_match.group(1).lower()
                self.entities_passed += 1

            # Define the entities
            if self.fill_match and self.stroke_match:
                self.entities = '\t<!ENTITY ' + self.stroke_entity + ' "' + \
                    self.stroke_color + '">\n'
                self.entities += '\t<!ENTITY ' + self.fill_entity + ' "' + \
                                 self.fill_color + '">\n'
                if self.use_iso_strokes:
                    self.entities += '\t<!ENTITY ' + self.iso_stroke_entity + \
                                     ' "' + self.stroke_color + '">\n'
            else:
                self.entities = '\t<!ENTITY ' + self.stroke_entity + ' "' + \
                    self.default_stroke_color + '">\n'
                self.entities += '\t<!ENTITY ' + self.fill_entity + ' "' + \
                                 self.default_fill_color + '">\n'
                if self.use_iso_strokes:
                    self.entities += '\t<!ENTITY ' + self.iso_stroke_entity + \
                                     ' "' + self.default_stroke_color + '">\n'

            # For simplicity, we simply replace the entire entity
            # declaration block; this obviously would remove any other
            # custom self.entities declared within the SVG, but we
            # assume that's an extreme edge case

            self.svgtext, self.n = \
                re.subn(r'(<!DOCTYPE[^>\[]*)(\[[^\]]*\])*\>',
                        r'\1 \n[\n' + self.entities + ']>\n', self.svgtext)

            # Add a doctype if none already exists, adding the
            # appropriate self.entities as well
            if self.n == 0:
                self.svgtext, self.n = \
                    re.subn('<svg',
                            "<!DOCTYPE svg  PUBLIC '-//W3C//DTD SVG 1.1//EN' \
'http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd' [\n" +
                            self.entities + "]>\n<svg", self.svgtext)
                if self.n == 0:
                    sys.exit('Error: Could not insert self.entities into DTD')

        # Convert self.entities to references
        self.stroke_entity = '&' + self.stroke_entity + ';'
        self.fill_entity = '&' + self.fill_entity + ';'

        # Create the SVG DOM
        try:
            self.svgxml = xml.dom.minidom.parseString(self.svgtext)
        except Exception as e:
            sys.exit('Error: Could not parse ' + self.svgfilename + str(e))

        # Extract top-level nodes
        self.i = 0
        self.svgindex = 0
        self.docindex = 0
        for element in self.svgxml.childNodes:
            if element.nodeType == 10:
                self.docindex = self.i
            elif element.localName == 'svg':
                self.svgindex = self.i
                break
            self.i += 1

        self.doctype = self.svgxml.childNodes[self.docindex]
        self.svg = self.svgxml.childNodes[self.svgindex]
        icons = self.svg.childNodes

        # Validate canvas size
        self.w = self.svg.getAttribute('width')
        self.h = self.svg.getAttribute('height')

        if self.w != '55px' or self.h != '55px':
            print 'Warning: invalid canvas size (%s, %s); \
Should be (55px, 55px)' % (self.w, self.h)

        # Guess the entity values, if they aren't passed in
        if self.use_entities:
            print 'entities_passed ==', self.entities_passed

            if self.entities_passed < 2:
                self.stroke_color, self.fill_color = \
                    self.guessEntities(self.svg)

            if self.confirm_guess or self.verbose:
                print '\nentity definitions:'
                print '     self.stroke_entity = ' + self.stroke_color
                print '     self.fill_entity = ' + self.fill_color

            '''if self.entities_passed < 2:
                if self.confirm_guess:
                    response = raw_input('\nAre these self.entities correct? [y/n] ')
                    if response.lower() != 'y':
                        print 'Please run this script again, passing the proper colors with the -s and -f flags.'
                        sys.exit(1)'''

        # Define the HTML for preview output
        self.previewHTML = "\
        <!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\"\n\
        \t\"http://www.w3.org/TR/html4/strict.dtd\">\n\
        <html>\n\
        <head>\n\
        \t<meta http-equiv=\"Content-type\" content=\"text/html; charset=utf-8\">\n\
        \t<title>Sugar Icon Preview: ~~~</title>\n\
        \t<script type=\"text/javascript\" charset=\"utf-8\">\n\
        \t\tvar bordered = false;\n\
        \t\tvar bgcolor = \"#FFF\"\n\
        \n\
        \t\tfunction toggleIconBorder(reset)\n\
        \t\t{\n\
        \t\t\tif(!reset) bordered = !bordered;\n\
        \t\t\t\n\
        \t\t\tvar objects = document.getElementsByTagName('object')\n\
        \t\t\tfor(var i = 0; i < objects.length; i++)\n\
        \t\t\t{\n\
        \t\t\t\tif(bordered) objects[i].style.border = 'solid 1px gray';\n\
        \t\t\t\telse objects[i].style.border = 'solid 1px ' + bgcolor;\n\
        \t\t\t}\n\
        \t\t}\n\
        \t\tfunction setBackgroundColor(color)\n\
        \t\t{\n\
        \t\t\tbgcolor = color;\n\
        \t\t\tvar objects = document.getElementsByTagName('div');\n\
        \t\t\tfor(var i = 0; i < objects.length; i++)\n\
        \t\t\t\tif(objects[i].className == 'cell')\n\
        \t\t\t\t\tobjects[i].style.backgroundColor = color;\n\
        \t\t\t\n\
        \t\t\ttoggleIconBorder(true)\n\
        \t\t}\n\
        \t</script>\n\
        \t<style type=\"text/css\" media=\"screen\">\n\
        \t\thtml, body {\n\
        \t\t\tmargin: 0px;\n\
        \t\t\tborder: 0px;\n\
        \t\t\tbackground-color: white;\n\
        \t\t\t\n\
        \t\t\tfont: 14px Helvetica;\n\
        \t\t\tcolor: gray;\n\
        \t\t}\n\
        \t\t.cell {\n\
        \t\t\twidth: 57px;\n\
        \t\t\theight: 57px;\n\
        \t\t\tpadding: 8px;\n\
        \t\t\tborder: solid 1px gray;\n\
        \t\t}\n\
        \t\t.icon {\n\
        \t\t\tmargin: 0px;\n\
        \t\t\tpadding: 0px;\n\
        \t\t\tborder: solid 1px #FFF;\n\
        \t\t\twidth: 55px;\n\
        \t\t\theight: 55px;\n\
        \t\t}\n\
        \t\t#icons {\n\
        \t\t\tmargin-top: 30px;\n\
        \t\t}\n\
        \n\
        \t\t#icons > li {\n\
        \t\t\tdisplay: block;\n\
        \t\t\tmargin: 0px;\n\
        \t\t\tmargin-bottom: 20px;\n\
        \t\t}\n\
        \t\t#description {\n\
        \t\t\twidth: 300px;\n\
        \t\t\tposition: absolute;\n\
        \t\t\ttop: 0px;\n\
        \t\t\tleft: 200px;\n\
        \t\t\tborder-left: solid 1px gray;\n\
        \t\t\tmargin-top: 30px;\n\
        \t\t\tpadding-left: 30px;\n\
        \t\t\tfont-size: 11px;\n\
        \t\t}\n\
        \t\t#description ul {\n\
        \t\t\tmargin: 0px;\n\
        \t\t\tpadding: 0px;\n\
        \t\t\tdisplay: block;\n\
        \t\t\tlist-style: square;\n\
        \t\t}\n\
        \t\tli { margin-bottom: 10px; }\n\
        \t\ta, a:visited { color: gray; }\n\
        \t\ta:hover { color: black; }\n\
        \t</style>\n\
        </head>\n\
        <body>\n\
        \t\t<ul id='icons'>\n\
        \t\t<li><div class='cell'><object class='icon' id='stroke' data=\"~~~.stroke.self.svg\" type=\"image/self.svg+xml\"></object></div><br>stroke\n\
        \t\t<li><div class='cell'><object class='icon' id='fill' data=\"~~~.fill.self.svg\" type=\"image/self.svg+xml\"></object></div><br>fill\n\
        \t\t<li><div class='cell'><object class='icon' id='both' data=\"~~~.both.self.svg\" type=\"image/self.svg+xml\"></object></div><br>both\n\
        \t\t</ul>\n\
        \t\t<div id='description'>\n\
        \t\t\t<h3>Icon Validation</h3>\n\
        \t\t\t<ul>\n\
        \t\t\t\t<li>Ensure that your icons appear to be centered within their boxes.\n\
        \t\t\t\t\tIf they appear off-center or cropped, you may have created your icon on canvas other than the required 55px square.\n\
        \t\t\t\t\tClick to <a href='javascript:toggleIconBorder();'>toggle</a> the 55px canvas border.\n\
        \t\t\t\t<li>If your icon appears off-center but has the correct 55px canvas, it may simply have uneven visual balance.\n\
        \t\t\t\t\tThis means, though it may be technically centered, differences in the distribution of \"mass\" cause it to appear otherwise.  \n\
        \t\t\t\t\tTry shifting the icon slightly on your canvas, while ensuring that you don't accidentally exceed the 55px boundary.\n\
        \t\t\t\t<!--li>Click to see your icon on <a href=\"javascript:setBackgroundColor('#000');\">black</a>, \n\
        \t\t\t\t\t<a href=\"javascript:setBackgroundColor('#FFF');\">white</a>, <a href=\"javascript:setBackgroundColor('#282828');\">gray</a>.<!-->\n\
        \t\t\t\t<li>Ensure that the first two icons appear entirely in gray, and that all of the third icon is colored blue and green, with the latter being the fill color.\n\
        \t\t\t\t\tIf any fail to meet these requirements, your icon does not have proper stroke and/or fill self.entities defined.\n\
        \t\t\t\t\tInvestigate the <b>-s</b> and <b>-f</b> options of sugar-iconify, and be sure that your input self.svg doesn't have extra colors in its palette.\n\
        \t\t\t\t<li>Ensure that your icon reads clearly when viewed only as strokes.\n\
        \t\t\t\t\tThis visual style will be used to represent activities/objects which are inactive, or uninstantiated.\n\
        \t\t\t\t\tConsider applying outlining strokes to any filled shapes that do not already have them.\n\
        \t\t\t\t<li>Ensure that your icon reads clearly when viewed only as fills.\n\
        \t\t\t\t\tThis visual style will be used for representing activity types within other icons, such as invitations, transfers, and objects.  \n\
        \t\t\t\t\tIf you have strokes which are isolated from fills, neither outlining them nor sitting against a filled background, please \n\
        \t\t\t\t\tinvestigate the <b>-i</b> option of the sugar-iconify script.\n\
        \t\t\t</ul>\n\
        \t\t\t<i>For more information, please see the OLPC wiki page on <a href='http://wiki.laptop.org/go/Making_Sugar_Icons' target='_blank'>making sugar icons</a>.</i>\n\
        \t\t</div>\n\
        </body>\n\
        </html>\n\
        "

        # Finally, do the icon conversion and export
        if self.multiple:
            # Export each icon as a separate file by top level group
            n_icons_exported = 0
            n_warnings = 0
            for icon in icons:

                try:
                    # Skip whitespace and unnamed icons
                    if icon.localName == 'g' and icon.attributes:

                        icon_name = ''
                        try:
                            if self.creator == 'inkscape' and \
                               icon.attributes.getNamedItem('inkscape:label'):
                                icon_name = icon.attributes.getNamedItem(
                                    'inkscape:label').nodeValue
                            else:
                                icon_name = icon.attributes.getNamedItem(
                                    'id').nodeValue
                        except BaseException:
                            pass

                        # Skip the template layers
                        if not icon_name.startswith('_'):

                            # Skip non-matches
                            if self.pattern == '' or \
                               re.search(self.pattern, icon_name):

                                if self.verbose:
                                    print '\nExporting ' + icon_name + \
                                        '.self.svg...'
                                icon_xml = xml.dom.minidom.Document()

                                # Construct the self.svg
                                icon_xml.appendChild(doctype)
                                icon_xml.appendChild(self.svg.cloneNode(0))

                                icon_xml.childNodes[1].appendChild(icon)
                                icon_xml.childNodes[1].childNodes[
                                    0].setAttribute('display', 'block')

                                if self.use_entities:
                                    strokes_replaced, fills_replaced = \
                                        self.replaceEntities(
                                            icon_xml.childNodes[1])

                                    if not strokes_replaced and not fills_replaced:
                                        print 'Warning: no entity replacements were made in %s' % icon_name
                                    elif not strokes_replaced:
                                        print 'Warning: no stroke entity replacements were made in %s' % icon_name
                                    elif not fills_replaced:
                                        print 'Warning: no fill entity replacements were made in %s' % icon_name

                                    if not strokes_replaced or not fills_replaced:
                                        n_warnings += 1

                                # Write the file
                                try:
                                    f = open(self.output_path + icon_name +
                                             '.self.svg', 'w')
                                except BaseException:
                                    sys.exit(
                                        'Error: Could not locate directory ' +
                                        self.output_path)

                                try:
                                    # Had to hack here to remove the
                                    # automatic encoding of '&' by
                                    # toxml() in entity refs I'm sure
                                    # there is a way to prevent need
                                    # for this if I knew the XML DOM
                                    # better
                                    icon_svgtext = icon_xml.toxml()
                                    icon_svgtext = re.sub('&amp;', '&',
                                                          icon_svgtext)
                                    if not self.use_default_colors:
                                        icon_svgtext = re.sub(
                                            r'ENTITY self.stroke_color "[^"]*"',
                                            r'ENTITY self.stroke_color "' +
                                            self.stroke_color + '"',
                                            icon_svgtext)
                                        icon_svgtext = re.sub(
                                            r'ENTITY self.fill_color "[^"]*"',
                                            r'ENTITY self.fill_color "' +
                                            self.fill_color +
                                            '"',
                                            icon_svgtext)
                                    f.write(icon_svgtext)
                                    f.close()
                                except BaseException:
                                    sys.exit(
                                        'Error: Could not write file ' +
                                        icon_name +
                                        '.self.svg')

                                n_icons_exported += 1
                except BaseException:
                    # Catch any errors we may have missed, so the rest
                    # of the icons can export normally
                    if(icon_name):
                        print 'Error: Could not export' + icon_name + \
                            '.self.svg'

            if self.verbose:
                if n_icons_exported == 1:
                    print 'Successfully exported 1 icon'
                else:
                    print 'Successfully exported %d icons' % n_icons_exported

                if n_warnings == 1:
                    print 'Warnings were reported for 1 icon'
                elif n_warnings > 1:
                    print 'Warnings were reported for %d icons' % n_warnings

        else:
            # Output a single converted icon
            if not self.overwrite_input:
                outfilename = re.sub(r'(.*\.)([^.]+)', r'\1sugar.\2',
                                     self.svgfilename)
                if self.verbose:
                    print 'Exporting ' + outfilename + ' ...'
            else:
                outfilename = self.svgfilename
                if self.verbose:
                    print 'Overwriting ' + outfilename + ' ...'

            # Remove the template layers
            for node in self.svg.childNodes:

                # Only check named nodes
                if node.localName == 'g' and node.attributes:
                    try:
                        if self.creator == 'inkscape' and \
                           node.attributes.getNamedItem('inkscape:label'):
                            node_name = node.attributes.getNamedItem(
                                'inkscape:label').nodeValue
                        else:
                            node_name = node.attributes.getNamedItem(
                                'id').nodeValue

                        if node_name.startswith('_'):
                            node.parentNode.removeChild(node)
                    except BaseException:
                        pass

            if self.use_entities:
                strokes_replaced, fills_replaced = \
                    self.replaceEntities(self.svgxml)
                if not strokes_replaced and not fills_replaced:
                    print 'Warning: no entity replacements were made'
                elif not strokes_replaced:
                    print 'Warning: no stroke entity replacements were made'
                elif not fills_replaced:
                    print 'Warning: no fill entity replacements were made'

                if self.use_iso_strokes:
                    strokes_fixed = self.fix_isolated_strokes(self.svgxml)
                    if strokes_fixed > 0 and self.verbose:
                        print "%d isolated strokes fixed" % strokes_fixed

            # Create the output file(s)
            if self.output_examples:

                example_path = self.output_path + \
                    re.sub(r'(.*\.)([^.]+)', r'\1preview',
                           self.svgfilename) + '/'
                try:
                    os.mkdir(example_path)
                except BaseException:
                    pass

                try:
                    f = open(example_path + 'preview.html', 'w')
                except BaseException:
                    print "Error: could not create HTML preview file"

                try:
                    f.write(re.sub(r'~~~', svgbasename, self.previewHTML))
                    f.close()
                except BaseException:
                    sys.exit('Error: could not write to HTML preview file')

                example_colors = [(self.default_stroke_color, '#FFFFFF',
                                   self.default_stroke_color),
                                  ('#FFFFFF', self.default_stroke_color,
                                   self.default_stroke_color),
                                  ('#0000AA', '#00DD00', '#0000AA')]
                example_filenames = [re.sub(r'(.*\.)([^.]+)', r'\1stroke.\2',
                                            self.svgfilename),
                                     re.sub(r'(.*\.)([^.]+)', r'\1fill.\2',
                                            self.svgfilename),
                                     re.sub(r'(.*\.)([^.]+)', r'\1both.\2',
                                            self.svgfilename)]

                icon_svgtext = self.svgxml.toxml()
                icon_svgtext = re.sub('&amp;', '&', icon_svgtext)

                for i in range(0, len(example_filenames)):
                    try:
                        f = open(example_path + example_filenames[i], 'w')
                    except BaseException:
                        sys.exit('Error: Could not save to ' + example_path +
                                 example_filenames[i])
                    try:
                        icon_svgtext = re.sub(
                            r'ENTITY self.stroke_color "[^"]*"',
                            r'ENTITY self.stroke_color "' +
                            example_colors[i][0] +
                            '"',
                            icon_svgtext)
                        icon_svgtext = re.sub(
                            r'ENTITY self.fill_color "[^"]*"',
                            r'ENTITY self.fill_color "' +
                            example_colors[i][1] +
                            '"',
                            icon_svgtext)
                        if self.use_iso_strokes:
                            icon_svgtext = re.sub(
                                r'ENTITY iso_stroke_color "[^"]*"',
                                r'ENTITY iso_stroke_color "' +
                                example_colors[i][2] +
                                '"',
                                icon_svgtext)
                        f.write(icon_svgtext)
                        f.close()
                    except BaseException:
                        sys.exit('Error: Could not write file ' +
                                 self.output_path + example_filenames[i])

            try:
                f = open(self.output_path + outfilename, 'w')
            except BaseException:
                sys.exit('Error: Could not save to ' + self.output_path +
                         outfilename)

            try:
                icon_svgtext = self.svgxml.toxml()
                icon_svgtext = re.sub('&amp;', '&', icon_svgtext)
                if not self.use_default_colors:
                    icon_svgtext = re.sub(
                        r'ENTITY self.stroke_color "[^"]*"',
                        r'ENTITY self.stroke_color "' +
                        self.stroke_color +
                        '"',
                        icon_svgtext)
                    icon_svgtext = re.sub(
                        r'ENTITY self.fill_color "[^"]*"',
                        r'ENTITY self.fill_color "' +
                        self.fill_color +
                        '"',
                        icon_svgtext)
                    if self.use_iso_strokes:
                        icon_svgtext = re.sub(
                            r'ENTITY iso_stroke_color "[^"]*"',
                            r'ENTITY iso_stroke_color "' +
                            self.stroke_color +
                            '"',
                            icon_svgtext)
                f.write(icon_svgtext)
                f.close()

            except BaseException:
                sys.exit('Error: Could not write file ' + self.output_path +
                         outfilename)

    # Define utility functions
    def getStroke(self, node):
        s = node.getAttribute('stroke')
        if s:
            return s.lower()
        else:
            if re.search(r'stroke:', node.getAttribute('style')):
                s = re.sub(
                    r'.*stroke:\s*(#*[^;]*).*',
                    r'\1',
                    node.getAttribute('style'))
                s2 = re.sub(
                    r'.*stroke:\s*rgb\(([^)]*).*',
                    r'\1',
                    node.getAttribute('style'))
                if 'rgb' not in s:
                    return s.lower()
                else:
                    percent_list = s2.split(',')
                    hex_str = '#'
                    for value in percent_list:
                        hex_str += self.percent_to_hex(value)
                    return hex_str
            else:
                return 'none'

    def setStroke(self, node, value):
        s = node.getAttribute('stroke')
        if s:
            node.setAttribute('stroke', value)
        else:
            s = re.sub(r'stroke:\s*[^;]*', 'stroke:' + value,
                       node.getAttribute('style'))
            node.setAttribute('style', s)

    def getFill(self, node):
        f = node.getAttribute('fill')
        if f:
            return f.lower()
        else:
            if re.search(r'fill:', node.getAttribute('style')):
                f = re.sub(r'.*fill:\s*(#*[^;]*).*', r'\1',
                           node.getAttribute('style'))
                f2 = re.sub(r'.*fill:\s*rgb\(([^)]*).*', r'\1',
                            node.getAttribute('style'))
                if 'rgb' not in f:
                    return f.lower()
                else:
                    percent_list = f2.split(',')
                    hex_str = '#'
                    for value in percent_list:
                        hex_str += self.percent_to_hex(value)
                    return hex_str
            else:
                return 'none'

    def setFill(self, node, value):
        f = node.getAttribute('fill')
        if f:
            node.setAttribute('fill', value)
        else:
            s = re.sub(r'fill:\s*[^;]*', 'fill:' + value,
                       node.getAttribute('style'))
            node.setAttribute('style', s)

    def replaceEntities(self, node, indent=''):

        strokes_replaced = 0
        fills_replaced = 0

        if node.localName:
            str = indent + node.localName

        if node.nodeType == 1:  # Only element nodes have attrs

            # Replace self.entities for matches
            if self.getStroke(node) == self.stroke_color:
                self.setStroke(node, self.stroke_entity)
                strokes_replaced += 1

            if self.getStroke(node) == self.fill_color:
                self.setStroke(node, self.fill_entity)
                strokes_replaced += 1

            if self.getFill(node) == self.fill_color:
                self.setFill(node, self.fill_entity)
                fills_replaced += 1

            if self.getFill(node) == self.stroke_color:
                self.setFill(node, self.stroke_entity)
                fills_replaced += 1

            str = str + " (" + self.getStroke(node) + ", " + \
                self.getFill(node) + ")"
            if self.verbose:
                print str

        # Recurse on DOM
        for n in node.childNodes:
            sr, fr = self.replaceEntities(n, indent + "   ")
            strokes_replaced += sr
            fills_replaced += fr

        # Return the number of replacements made
        return (strokes_replaced, fills_replaced)

    def fix_isolated_strokes(self, node):
        strokes_fixed = 0
        # Recurse on DOM
        last_n = None
        for n in node.childNodes:
            sf = self.fix_isolated_strokes(n)
            strokes_fixed += sf

        if node.nodeType == 1:  # Only element nodes have attrs

            # Find strokes with no associated fill
            if self.getStroke(node) != 'none' and self.getFill(node) == 'none':
                strokes_fixed += 1
                self.setStroke(node, "&iso_stroke_color;")

        # Return the number of strokes fixed
        return strokes_fixed

    # These functions attempt to guess the hex values for the stroke
    # and fill self.entities

    def getColorPairs(self, node, pairs=[]):
        if node.nodeType == 1:

            # Skip masks
            if node.localName == 'mask':
                return pairs

            node_name = ''
            try:
                if self.creator == 'inkscape' and \
                   node.attributes.getNamedItem('inkscape:label'):
                    node_name = node.attributes.getNamedItem(
                        'inkscape:label').nodeValue
                else:
                    node_name = node.attributes.getNamedItem('id').nodeValue
            except BaseException:
                pass

            # Skip the template layers
            if node_name.startswith('_'):
                return pairs

            pair = (self.getStroke(node), self.getFill(node))
            if pair[0] != pair[1]:
                pairs.append(pair)

        # Recurse on DOM
        for n in node.childNodes:
            self.getColorPairs(n, pairs)

        return pairs

    def guessEntities(self, node):
        guesses = self.getColorPairs(node)

        if self.stroke_color is not None:
            stroke_guess = self.stroke_color
        else:
            stroke_guess = 'none'

        if self.stroke_color is not None:
            fill_guess = self.fill_color
        else:
            fill_guess = 'none'

        for guess in guesses:
            if stroke_guess == 'none':
                stroke_guess = guess[0]
            if fill_guess == 'none' and stroke_guess != guess[1]:
                fill_guess = guess[1]
            if guess[0] == fill_guess and guess[1] != 'none':
                fill_guess = stroke_guess
                stroke_guess = guess[0]
                if fill_guess == 'none':
                    fill_guess = guess[1]

        return (stroke_guess, fill_guess)


if __name__ == '__main__':
    SugarIconify(command_line=True)



# -*- coding: utf-8 -*-
# Copyright (c) 2012, Walter Bender
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import tempfile
from time import time
from gettext import gettext as _

from plugins.plugin import Plugin
from TurtleArt.tapalette import (make_palette, define_logo_function)
from TurtleArt.talogo import (primitive_dictionary, logoerror)
from TurtleArt.taconstants import (CONSTANTS, MACROS, KEY_DICT, MEDIA_SHAPES,
                                   REVERSE_KEY_DICT, SKIN_PATHS,
                                   BLOCKS_WITH_SKIN, PYTHON_SKIN,
                                   MEDIA_BLOCK2TYPE, VOICES)
from TurtleArt.tautils import (debug_output, get_path, data_to_string,
                               hat_on_top, listify, data_from_file,
                               get_endswith_files)
from TurtleArt.taprimitive import (ArgSlot, ConstantArg, Primitive)
from TurtleArt.tatype import (TYPE_BOOL, TYPE_BOX, TYPE_CHAR, TYPE_INT,
                              TYPE_FLOAT, TYPE_OBJECT, TYPE_STRING,
                              TYPE_NUMBER)
from TurtleArt.taturtle import Turtle


class Turtle_blocks_extras(Plugin):
    """ a class for defining the extra palettes that distinguish Turtle Blocks
    from Turtle Art """

    def __init__(self, turtle_window):
        Plugin.__init__(self)
        self.tw = turtle_window

    def setup(self):
        SKIN_PATHS.append('plugins/turtle_blocks_extras/images')

        self.heap = self.tw.lc.heap
        self.title_height = int((self.tw.canvas.height / 20) * self.tw.scale)

        # set up Turtle Block palettes
        self._flow_palette()

        self._sensor_palette()

        self._media_palette()

        self._extras_palette()

        self._portfolio_palette()

        self._myblocks_palette()

    # Palette definitions

    def _flow_palette(self):
        palette = make_palette('flow',
                               colors=["#FFC000", "#A08000"],
                               help_string=_('Palette of flow operators'))

        palette.add_block('while',
                          style='clamp-style-boolean',
                          label=_('while'),
                          prim_name='while',
                          default=[None, None],
                          special_name=_('while'),
                          help_string=_('do-while-True operator that uses \
boolean operators from Numbers palette'))
        self.tw.lc.def_prim(
            'while', 2,
            Primitive(self.tw.lc.prim_loop,
                      arg_descs=[
                          ArgSlot(TYPE_OBJECT,
                                  call_arg=False,
                                  wrapper=Primitive(
                                      Primitive.controller_while,
                                      arg_descs=[ArgSlot(TYPE_BOOL,
                                                         call_arg=False)])),
                          ArgSlot(TYPE_OBJECT)]),
            True)

        palette.add_block('until',
                          style='clamp-style-until',
                          label=_('until'),
                          prim_name='until',
                          default=[None, None],
                          special_name=_('until'),
                          help_string=_('do-until-True operator that uses \
boolean operators from Numbers palette'))
        self.tw.lc.def_prim(
            'until', 2,
            Primitive(self.tw.lc.prim_loop,
                      arg_descs=[
                          ArgSlot(TYPE_OBJECT,
                                  call_arg=False,
                                  # TODO can we use controller_while in
                                  # combination with not_?
                                  wrapper=Primitive(
                                      Primitive.controller_until,
                                      arg_descs=[ArgSlot(TYPE_BOOL,
                                                         call_arg=False)])),
                          ArgSlot(TYPE_OBJECT)]),
            True)

        palette.add_block('sandwichclamp',
                          style='clamp-style-collapsible',
                          label=' ',
                          special_name=_('top'),
                          prim_name='clamp',
                          help_string=_('top of a collapsible stack'))
        self.tw.lc.def_prim('clamp', 1,
                            Primitive(self.tw.lc.prim_clamp,
                                      arg_descs=[ArgSlot(TYPE_OBJECT)]),
                            True)

    def _media_palette(self):

        palette = make_palette('media',
                               colors=["#A0FF00", "#80A000"],
                               help_string=_('Palette of media objects'),
                               position=7,
                               translation=_('media'))

        palette.add_block('journal',
                          style='box-style-media',
                          label=' ',
                          default='None',
                          special_name=_('journal'),
                          help_string=_('Sugar Journal media object'))
        MEDIA_BLOCK2TYPE['journal'] = 'media'
        BLOCKS_WITH_SKIN.append('journal')
        MEDIA_SHAPES.append('journalsmall')
        MEDIA_SHAPES.append('journaloff')
        MEDIA_SHAPES.append('journalon')

        palette.add_block('audio',
                          hidden=True,
                          style='box-style-media',
                          label=' ',
                          special_name=_('audio'),
                          default='None',
                          help_string=_('Sugar Journal audio object'))
        BLOCKS_WITH_SKIN.append('audio')
        MEDIA_BLOCK2TYPE['audio'] = 'audio'
        MEDIA_SHAPES.append('audiosmall')
        MEDIA_SHAPES.append('audiooff')
        MEDIA_SHAPES.append('audioon')

        palette.add_block('video',
                          hidden=True,
                          style='box-style-media',
                          label=' ',
                          special_name=_('video'),
                          default='None',
                          help_string=_('Sugar Journal video object'))
        BLOCKS_WITH_SKIN.append('video')
        MEDIA_BLOCK2TYPE['video'] = 'video'
        MEDIA_SHAPES.append('videosmall')
        MEDIA_SHAPES.append('videooff')
        MEDIA_SHAPES.append('videoon')

        palette.add_block('description',
                          hidden=True,
                          style='box-style-media',
                          label=' ',
                          special_name=_('description'),
                          default='None',
                          help_string=_('Sugar Journal description field'))
        BLOCKS_WITH_SKIN.append('description')
        MEDIA_BLOCK2TYPE['description'] = 'descr'
        MEDIA_SHAPES.append('descriptionsmall')
        MEDIA_SHAPES.append('descriptionoff')
        MEDIA_SHAPES.append('descriptionon')

        palette.add_block('string',
                          style='box-style',
                          label=_('text'),
                          default=_('text'),
                          special_name=_('text'),
                          help_string=_('string value'))

        palette.add_block('show',
                          style='basic-style-1arg',
                          label=_('show'),
                          default=_('text'),
                          prim_name='show',
                          logo_command='label',
                          help_string=_('draws text or show media from the \
Journal'))
        self.tw.lc.def_prim('show', 1,
                            Primitive(self.tw.lc.show,
                                      arg_descs=[ArgSlot(TYPE_OBJECT),
                                                 ConstantArg(True)]))

        palette.add_block('showaligned',
                          hidden=True,
                          colors=["#A0FF00", "#80A000"],
                          style='basic-style-1arg',
                          label=_('show aligned'),
                          default=_('text'),
                          prim_name='showaligned',
                          logo_command='label',
                          help_string=_('draws text or show media from the \
Journal'))
        self.tw.lc.def_prim('showaligned', 1,
                            Primitive(self.tw.lc.show,
                                      arg_descs=[ArgSlot(TYPE_OBJECT),
                                                 ConstantArg(False)]))

        palette.add_block('setscale',
                          style='basic-style-1arg',
                          label=_('set scale'),
                          prim_name='setscale',
                          default=33,
                          logo_command='setlabelheight',
                          help_string=_('sets the scale of media'))
        self.tw.lc.def_prim(
            'setscale', 1,
            Primitive(self.tw.lc.set_scale,
                      arg_descs=[ArgSlot(TYPE_NUMBER)],
                      call_afterwards=lambda value: self.after_set(
                          'scale', value)))

        palette.add_block(
            'setfont',
            style='basic-style-1arg',
            label=_('set font'),
            prim_name='setfont',
            default='Sans',
            help_string=_('sets the font used by the show block'))
        self.tw.lc.def_prim('setfont', 1,
                            Primitive(self.tw.canvas.set_font,
                                      arg_descs=[ArgSlot(TYPE_STRING)]))

        palette.add_block('savepix',
                          style='basic-style-1arg',
                          label=_('save picture'),
                          prim_name='savepix',
                          default=_('picture name'),
                          help_string=_('saves a picture to the Sugar \
Journal'))
        self.tw.lc.def_prim('savepix', 1,
                            Primitive(self.tw.save_as_image,
                                      arg_descs=[ArgSlot(TYPE_STRING)]))

        palette.add_block('savesvg',
                          style='basic-style-1arg',
                          label=_('save SVG'),
                          prim_name='savesvg',
                          default=_('picture name'),
                          help_string=_('saves turtle graphics as an SVG file \
in the Sugar Journal'))
        self.tw.lc.def_prim('savesvg', 1,
                            Primitive(self.tw.save_as_image,
                                      arg_descs=[ArgSlot(TYPE_STRING)],
                                      kwarg_descs={'svg': ConstantArg(True)}))

        palette.add_block('scale',
                          style='box-style',
                          label=_('scale'),
                          prim_name='scale',
                          value_block=True,
                          logo_command='labelsize',
                          help_string=_('holds current scale value'))
        self.tw.lc.def_prim('scale', 0,
                            Primitive(self.tw.lc.get_scale,
                                      return_type=TYPE_NUMBER))

        palette.add_block('mediawait',
                          style='basic-style-extended-vertical',
                          label=_('media wait'),
                          prim_name='mediawait',
                          help_string=_('wait for current video or audio to \
complete'))
        self.tw.lc.def_prim('mediawait', 0, self.tw.lc.media_wait, True)

        palette.add_block('mediastop',
                          style='basic-style-extended-vertical',
                          label=_('media stop'),
                          prim_name='mediastop',
                          help_string=_('stop video or audio'))
        self.tw.lc.def_prim('mediastop', 0, self.tw.lc.media_stop, True)

        palette.add_block('mediapause',
                          style='basic-style-extended-vertical',
                          label=_('media pause'),
                          prim_name='mediapause',
                          help_string=_('pause video or audio'))
        self.tw.lc.def_prim('mediapause', 0, self.tw.lc.media_pause, True)

        palette.add_block('mediaplay',
                          style='basic-style-extended-vertical',
                          label=_('media resume'),
                          prim_name='mediaplay',
                          help_string=_('resume playing video or audio'))
        self.tw.lc.def_prim('mediaplay', 0, self.tw.lc.media_play, True)

        palette.add_block('speak',
                          style='basic-style-1arg',
                          label=_('speak'),
                          prim_name='speak',
                          default=_('hello'),
                          help_string=_('speaks text'))
        self.tw.lc.def_prim('speak', 1,
                            Primitive(self.prim_speak,
                                      arg_descs=[ArgSlot(TYPE_STRING)]))

        palette.add_block('sinewave',
                          style='basic-style-3arg',
                          # TRANS: pitch, duration, amplitude
                          label=[_('sinewave') + '\n\n', _('pitch'),
                                 _('amplitude'), _('duration')],
                          prim_name='sinewave',
                          default=[1000, 5000, 1],
                          help_string=_('plays a sinewave at frequency, \
amplitude, and duration (in seconds)'))
        self.tw.lc.def_prim('sinewave', 3,
                            Primitive(self.prim_sinewave,
                                      arg_descs=[ArgSlot(TYPE_NUMBER),
                                                 ArgSlot(TYPE_NUMBER),
                                                 ArgSlot(TYPE_NUMBER)]))

    def _sensor_palette(self):

        palette = make_palette('sensor',
                               colors=["#FF6060", "#A06060"],
                               help_string=_('Palette of sensor blocks'),
                               position=6,
                               translation=_('sensor'))

        palette.add_block('mousebutton',
                          hidden=True,
                          style='box-style',
                          label=_('button down'),
                          prim_name='mousebutton',
                          value_block=True,
                          help_string=_('returns 1 if mouse button is \
pressed'))
        self.tw.lc.def_prim('mousebutton', 0,
                            Primitive(self.tw.get_mouse_flag,
                                      return_type=TYPE_NUMBER))

        palette.add_block('mousebutton2',
                          style='boolean-block-style',
                          label=_('button down'),
                          prim_name='mousebutton2',
                          value_block=True,
                          help_string=_('returns True if mouse button is \
pressed'))
        self.tw.lc.def_prim('mousebutton2', 0,
                            Primitive(self.tw.get_mouse_button,
                                      return_type=TYPE_BOOL))

        palette.add_block('mousex',
                          style='box-style',
                          label=_('mouse x'),
                          prim_name='mousex',
                          value_block=True,
                          help_string=_('returns mouse x coordinate'))
        self.tw.lc.def_prim('mousex', 0,
                            Primitive(self.tw.get_mouse_x,
                                      return_type=TYPE_NUMBER,
                                      call_afterwards=self.after_mouse_x))

        palette.add_block('mousey',
                          style='box-style',
                          label=_('mouse y'),
                          prim_name='mousey',
                          value_block=True,
                          help_string=_('returns mouse y coordinate'))
        self.tw.lc.def_prim('mousey', 0,
                            Primitive(self.tw.get_mouse_y,
                                      return_type=TYPE_NUMBER,
                                      call_afterwards=self.after_mouse_y))

        palette.add_block('kbinput',
                          style='basic-style-extended-vertical',
                          label=_('query keyboard'),
                          prim_name='kbinput',
                          help_string=_('query for keyboard input (results \
stored in keyboard block)'))
        self.tw.lc.def_prim('kbinput', 0,
                            Primitive(self.tw.get_keyboard_input,
                                      call_afterwards=self.after_keypress))

        palette.add_block('keyboard',
                          style='box-style',
                          label=_('keyboard'),
                          prim_name='keyboard',
                          value_block=True,
                          logo_command='make "keyboard readchar',
                          help_string=_('holds results of query-keyboard \
block as ASCII'))
        self.tw.lc.def_prim('keyboard', 0,
                            Primitive(self.tw.get_keyboard,
                                      return_type=TYPE_NUMBER))

        palette.add_block('readpixel',
                          style='basic-style-extended-vertical',
                          label=_('read pixel'),
                          prim_name='readpixel',
                          logo_command=':keyboard',
                          help_string=_('RGB color under the turtle is pushed \
to the stack'))
        self.tw.lc.def_prim('readpixel', 0,
                            Primitive(Turtle.read_pixel))

        palette.add_block('see',
                          style='box-style',
                          label=_('turtle sees'),
                          value_block=True,
                          prim_name='see',
                          help_string=_('returns the color that the turtle \
"sees"'))
        self.tw.lc.def_prim('see', 0,
                            Primitive(Turtle.get_color_index,
                                      return_type=TYPE_NUMBER,
                                      call_afterwards=self.after_see))

        palette.add_block('time',
                          style='box-style',
                          label=_('time'),
                          prim_name='time',
                          value_block=True,
                          help_string=_('elapsed time (in seconds) since \
program started'))
        self.tw.lc.def_prim(
            'time', 0,
            Primitive(
                Primitive.identity,
                return_type=TYPE_INT,
                arg_descs=[
                    ConstantArg(
                        Primitive(
                            int,
                            arg_descs=[ConstantArg(
                                Primitive(Primitive.minus,
                                          arg_descs=[
                                              ConstantArg(Primitive(time)),
                                              ConstantArg(Primitive(
                                                  self.tw.lc.get_start_time))])
                            )]
                        ))],
                call_afterwards=self.after_time))

    def _extras_palette(self):

        palette = make_palette('extras',
                               colors=["#FF0000", "#A00000"],
                               help_string=_('Palette of extra options'),
                               position=8,
                               translation=_('extras'))

        palette.add_block('push',
                          style='basic-style-1arg',
                          # TRANS: push adds a new item to the program stack
                          label=_('push'),
                          prim_name='push',
                          logo_command='tapush',
                          help_string=_('pushes value onto FILO (first-in \
last-out heap)'))
        self.tw.lc.def_prim(
            'push', 1,
            Primitive(self.tw.lc.append_heap,
                      arg_descs=[ArgSlot(TYPE_OBJECT)],
                      call_afterwards=self.after_push))
        define_logo_function('tapush', 'to tapush :foo\nmake "taheap fput \
:foo :taheap\nend\nmake "taheap []\n')

        palette.add_block('printheap',
                          style='basic-style-extended-vertical',
                          label=_('show heap'),
                          prim_name='printheap',
                          logo_command='taprintheap',
                          help_string=_('shows values in FILO (first-in \
last-out heap)'))
        self.tw.lc.def_prim(
            'printheap', 0,
            Primitive(self.tw.print_,
                      arg_descs=[ConstantArg(Primitive(self.tw.lc.get_heap)),
                                 ConstantArg(False)]))
        define_logo_function('taprintheap', 'to taprintheap \nprint :taheap\n\
end\n')

        palette.add_block('clearheap',
                          style='basic-style-extended-vertical',
                          label=_('empty heap'),
                          prim_name='clearheap',
                          logo_command='taclearheap',
                          help_string=_('emptys FILO (first-in-last-out \
heap)'))
        self.tw.lc.def_prim(
            'clearheap', 0,
            Primitive(self.tw.lc.reset_heap, call_afterwards=self.after_pop))
        define_logo_function('taclearheap', 'to taclearheap\nmake "taheap []\n\
end\n')

        palette.add_block('pop',
                          style='box-style',
                          # TRANS: pop removes a new item from the program
                          # stack
                          label=_('pop'),
                          prim_name='pop',
                          value_block=True,
                          logo_command='tapop',
                          help_string=_('pops value off FILO (first-in \
last-out heap)'))
        self.tw.lc.def_prim(
            'pop', 0,
            Primitive(self.tw.lc.pop_heap, return_type=TYPE_BOX,
                      call_afterwards=self.after_pop))
        define_logo_function('tapop', 'to tapop\nif emptyp :taheap [stop]\n\
make "tmp first :taheap\nmake "taheap butfirst :taheap\noutput :tmp\nend\n')

        palette.add_block('isheapempty',
                          hidden=True,
                          style='box-style',
                          label=_('empty heap?'),
                          prim_name='isheapempty',
                          value_block=True,
                          help_string=_('returns True if heap is empty'))
        self.tw.lc.def_prim(
            'isheapempty', 0,
            Primitive(int, return_type=TYPE_INT,
                      arg_descs=[ConstantArg(
                          Primitive(Primitive.not_, return_type=TYPE_BOOL,
                                    arg_descs=[ConstantArg(
                                        Primitive(self.tw.lc.get_heap,
                                                  return_type=TYPE_BOOL))]))]))

        palette.add_block('saveheap',
                          style='basic-style-1arg',
                          label=_('save heap to file'),
                          default=_('filename'),
                          prim_name='saveheap',
                          help_string=_('saves FILO (first-in \
last-out heap) to a file'))

        self.tw.lc.def_prim('saveheap', 1,
                            Primitive(self.tw.lc.save_heap,
                                      arg_descs=[ArgSlot(TYPE_OBJECT)]))

        if self.tw.running_sugar:
            palette.add_block('loadheap',
                              style='basic-style-1arg',
                              hidden=True,
                              label=_('load heap from file'),
                              default=_('filename'),
                              prim_name='loadheap',
                              help_string=_('loads FILO (first-in \
last-out heap) from a file'))
            # macro
            palette.add_block('loadheapfromjournal',
                              style='basic-style-1arg',
                              label=_('load heap from file'),
                              help_string=_('loads FILO (first-in \
last-out heap) from a file'))
        else:
            palette.add_block('loadheap',
                              style='basic-style-1arg',
                              label=_('load heap from file'),
                              default=_('filename'),
                              prim_name='loadheap',
                              help_string=_('loads FILO (first-in \
last-out heap) from a file'))

        self.tw.lc.def_prim('loadheap', 1,
                            Primitive(self.tw.lc.load_heap,
                                      arg_descs=[ArgSlot(TYPE_OBJECT)],
                                      call_afterwards=self.after_push))

        palette.add_block('isheapempty2',
                          style='boolean-block-style',
                          label=_('empty heap?'),
                          prim_name='isheapempty2',
                          value_block=True,
                          help_string=_('returns True if heap is empty'))
        self.tw.lc.def_prim(
            'isheapempty2', 0,
            # Python automatically converts the heap to a boolean in contexts
            # where a boolean is needed
            Primitive(Primitive.not_, return_type=TYPE_BOOL,
                      arg_descs=[ConstantArg(
                          Primitive(self.tw.lc.get_heap,
                                    return_type=TYPE_BOOL))]))

        palette.add_block('comment',
                          style='basic-style-1arg',
                          label=_('comment'),
                          prim_name='comment',
                          default=_('comment'),
                          string_or_number=True,
                          help_string=_('places a comment in your code'))
        self.tw.lc.def_prim(
            'comment', 1,
            Primitive(Primitive.comment, arg_descs=[ArgSlot(TYPE_STRING)]))

        palette.add_block('print',
                          style='basic-style-1arg',
                          label=_('print'),
                          prim_name='print',
                          logo_command='label',
                          string_or_number=True,
                          help_string=_('prints value in status block at \
bottom of the screen'))
        self.tw.lc.def_prim(
            'print', 1,
            Primitive(self.tw.print_,
                      arg_descs=[ArgSlot(TYPE_OBJECT), ConstantArg(False)]))

        palette.add_block('chr',
                          style='number-style-1arg',
                          label='chr',
                          prim_name='chr',
                          help_string=_('Python chr operator'))
        self.tw.lc.def_prim(
            'chr', 1,
            Primitive(chr, return_type=TYPE_CHAR,
                      arg_descs=[ArgSlot(TYPE_INT)]))

        palette.add_block('int',
                          style='number-style-1arg',
                          label='int',
                          prim_name='int',
                          help_string=_('Python int operator'))
        self.tw.lc.def_prim(
            'int', 1,
            # leave over the actual work to the type system, and just demand
            # that the argument be converted to an integer
            Primitive(Primitive.identity, return_type=TYPE_INT,
                      arg_descs=[ArgSlot(TYPE_INT)]))

        palette.add_block('polar',
                          style='basic-style-extended-vertical',
                          label=_('polar'),
                          prim_name='polar',
                          help_string=_('displays polar coordinates'))
        self.tw.lc.def_prim('polar', 0,
                            lambda self: self.tw.set_polar(True))

        palette.add_block('myfunc1arg',
                          style='number-style-var-arg',
                          label=[_('Python'), 'f(x)', 'x'],
                          prim_name='myfunction',
                          default=['x', 100],
                          string_or_number=True,
                          help_string=_('a programmable block: used to add \
advanced single-variable math equations, e.g., sin(x)'))
        self.tw.lc.def_prim(
            'myfunction', 2,
            Primitive(self.tw.lc.prim_myfunction, return_type=TYPE_FLOAT,
                      arg_descs=[ArgSlot(TYPE_STRING), ArgSlot(TYPE_FLOAT)]))

        palette.add_block('myfunc2arg',
                          hidden=True,
                          colors=["#FF0000", "#A00000"],
                          style='number-style-var-arg',
                          label=[_('Python'), 'f(x,y)', ' '],
                          prim_name='myfunction2',
                          default=['x+y', 100, 100],
                          string_or_number=True,
                          help_string=_('a programmable block: used to add \
advanced multi-variable math equations, e.g., sqrt(x*x+y*y)'))
        self.tw.lc.def_prim(
            'myfunction2', 3,
            Primitive(self.tw.lc.prim_myfunction, return_type=TYPE_FLOAT,
                      arg_descs=[ArgSlot(TYPE_STRING), ArgSlot(TYPE_FLOAT),
                                 ArgSlot(TYPE_FLOAT)]))

        palette.add_block('myfunc3arg',
                          hidden=True,
                          colors=["#FF0000", "#A00000"],
                          style='number-style-var-arg',
                          label=[_('Python'), 'f(x,y,z)', ' '],
                          prim_name='myfunction3',
                          default=['x+y+z', 100, 100, 100],
                          string_or_number=True,
                          help_string=_('a programmable block: used to add \
advanced multi-variable math equations, e.g., sin(x+y+z)'))
        self.tw.lc.def_prim(
            'myfunction3', 4,
            Primitive(self.tw.lc.prim_myfunction, return_type=TYPE_FLOAT,
                      arg_descs=[ArgSlot(TYPE_STRING), ArgSlot(TYPE_FLOAT),
                                 ArgSlot(TYPE_FLOAT), ArgSlot(TYPE_FLOAT)]))

        palette.add_block('cartesian',
                          style='basic-style-extended-vertical',
                          label=_('Cartesian'),
                          prim_name='cartesian',
                          help_string=_('displays Cartesian coordinates'))
        self.tw.lc.def_prim('cartesian', 0,
                            lambda self: self.tw.set_cartesian(True))

        palette.add_block('userdefined',
                          style='basic-style-var-arg',
                          label=' ',
                          prim_name='userdefined',
                          string_or_number=True,
                          special_name=_('Python block'),
                          default=100,
                          help_string=_('runs code found in the tamyblock.py \
module found in the Journal'))
        self.tw.lc.def_prim('userdefined', 1,
                            Primitive(self.tw.lc.prim_myblock,
                                      arg_descs=[ArgSlot(TYPE_OBJECT)]))
        BLOCKS_WITH_SKIN.append('userdefined')
        PYTHON_SKIN.append('userdefined')

        palette.add_block('userdefined2args',
                          hidden=True,
                          colors=["#FF0000", "#A00000"],
                          style='basic-style-var-arg',
                          label=' ',
                          prim_name='userdefined2',
                          string_or_number=True,
                          special_name=_('Python block'),
                          default=[100, 100],
                          help_string=_('runs code found in the tamyblock.py \
module found in the Journal'))
        self.tw.lc.def_prim('userdefined2', 2,
                            Primitive(self.tw.lc.prim_myblock,
                                      arg_descs=[ArgSlot(TYPE_OBJECT),
                                                 ArgSlot(TYPE_OBJECT)]))
        BLOCKS_WITH_SKIN.append('userdefined2args')
        PYTHON_SKIN.append('userdefined2args')

        palette.add_block('userdefined3args',
                          hidden=True,
                          colors=["#FF0000", "#A00000"],
                          style='basic-style-var-arg',
                          label=' ',
                          prim_name='userdefined3',
                          special_name=_('Python block'),
                          default=[100, 100, 100],
                          string_or_number=True,
                          help_string=_('runs code found in the tamyblock.py \
module found in the Journal'))
        self.tw.lc.def_prim('userdefined3', 3,
                            Primitive(self.tw.lc.prim_myblock,
                                      arg_descs=[ArgSlot(TYPE_OBJECT),
                                                 ArgSlot(TYPE_OBJECT),
                                                 ArgSlot(TYPE_OBJECT)]))
        BLOCKS_WITH_SKIN.append('userdefined3args')
        PYTHON_SKIN.append('userdefined3args')
        MEDIA_SHAPES.append('pythonsmall')
        MEDIA_SHAPES.append('pythonoff')
        MEDIA_SHAPES.append('pythonon')

        palette.add_block('getfromurl',
                          style='number-style-1arg',
                          # TRANS: URL is universal resource locator
                          label=_('URL'),
                          default='http://wiki.sugarlabs.org/images/2/2c/Logo_alt_3.svg',
                          prim_name='getfromurl',
                          help_string=_('gets a text string or an image from a URL'))
        self.tw.lc.def_prim('getfromurl', 1,
                            Primitive(self.tw.lc.get_from_url,
                                      arg_descs=[ArgSlot(TYPE_STRING)]))

        palette.add_block('skin',
                          hidden=True,
                          colors=["#FF0000", "#A00000"],
                          style='basic-style-1arg',
                          label=_('turtle shell'),
                          prim_name='skin',
                          help_string=_("put a custom 'shell' on the turtle"))
        self.tw.lc.def_prim('skin', 1,
                            Primitive(self.tw.lc.reskin,
                                      arg_descs=[ArgSlot(TYPE_OBJECT)]))

        # macro
        palette.add_block('reskin',
                          style='basic-style-1arg',
                          label=_('turtle shell'),
                          help_string=_("put a custom 'shell' on the turtle"))

        palette.add_block('addturtle',
                          style='basic-style-1arg',
                          label=_('turtle'),
                          prim_name='addturtle',
                          default=1,
                          string_or_number=True,
                          help_string=_('chooses which turtle to command'))
        self.tw.lc.def_prim('addturtle', 1,
                            Primitive(self.tw.lc.prim_turtle,
                                      arg_descs=[ArgSlot(TYPE_STRING)]))

        palette.add_block('turtlex',
                          style='number-style-1arg',
                          label=_('turtle x'),
                          prim_name='turtlex',
                          default=['Yertle'],
                          help_string=_('Returns x coordinate of turtle'))
        self.tw.lc.def_prim(
            'turtlex', 1,
            Primitive(self.tw.turtles.get_turtle_x,
                      arg_descs=[ArgSlot(TYPE_OBJECT)],
                      return_type=TYPE_BOX))

        palette.add_block('turtley',
                          style='number-style-1arg',
                          label=_('turtle y'),
                          prim_name='turtley',
                          default=['Yertle'],
                          help_string=_('Returns y coordinate of turtle'))
        self.tw.lc.def_prim(
            'turtley', 1,
            Primitive(self.tw.turtles.get_turtle_y,
                      arg_descs=[ArgSlot(TYPE_OBJECT)],
                      return_type=TYPE_BOX))

        palette.add_block('activeturtle',
                          style='box-style',
                          label=_('active turtle'),
                          prim_name='activeturtle',
                          value_block=True,
                          help_string=_('the name of the active turtle'))
        self.tw.lc.def_prim(
            'activeturtle', 0,
            Primitive(Turtle.get_name,
                      return_type=TYPE_BOX))

        palette.add_block('turtleh',
                          style='number-style-1arg',
                          label=_('turtle heading'),
                          prim_name='turtleh',
                          default=['Yertle'],
                          help_string=_('Returns heading of turtle'))
        self.tw.lc.def_prim(
            'turtleh', 1,
            Primitive(self.tw.turtles.get_turtle_heading,
                      arg_descs=[ArgSlot(TYPE_OBJECT)],
                      return_type=TYPE_BOX))

        palette.add_block('sandwichclampcollapsed',
                          hidden=True,
                          style='clamp-style-collapsed',
                          label=_('click to open'),
                          prim_name='clamp',
                          special_name=_('top'),
                          help_string=_('top of a collapsed stack'))

        palette.add_block('loadpalette',
                          style='basic-style-1arg',
                          string_or_number=True,
                          label=_('select palette'),
                          prim_name='loadpalette',
                          default=_('turtle'),
                          help_string=_('selects a palette'))
        self.tw.lc.def_prim('loadpalette', 1,
                            Primitive(self.tw.prim_load_palette,
                                      export_me=False,
                                      arg_descs=[ArgSlot(TYPE_STRING)]))

        palette.add_block('loadblock',
                          style='basic-style-var-arg',
                          label=_('load'),
                          prim_name='loadblock',
                          default=_('forward'),
                          help_string=_('loads a block'))
        self.tw.lc.def_prim('loadblock', 1,
                            Primitive(self.tw.prim_load_block,
                                      export_me=False,
                                      arg_descs=[ArgSlot(TYPE_STRING)]))

        palette.add_block('loadblock2arg',
                          style='basic-style-var-arg',
                          hidden=True,
                          label=_('load'),
                          prim_name='loadblock2',
                          string_or_number=True,
                          default=[_('forward'), 100],
                          help_string=_('loads a block'))
        self.tw.lc.def_prim('loadblock2', 2,
                            Primitive(self.tw.prim_load_block,
                                      export_me=False,
                                      arg_descs=[ArgSlot(TYPE_STRING),
                                                 ArgSlot(TYPE_OBJECT)]))

        palette.add_block('loadblock3arg',
                          style='basic-style-var-arg',
                          hidden=True,
                          label=_('load'),
                          string_or_number=True,
                          prim_name='loadblock3',
                          default=[_('setxy'), 0, 0],
                          help_string=_('loads a block'))
        self.tw.lc.def_prim('loadblock3', 3,
                            Primitive(self.tw.prim_load_block,
                                      export_me=False,
                                      arg_descs=[ArgSlot(TYPE_STRING),
                                                 ArgSlot(TYPE_OBJECT),
                                                 ArgSlot(TYPE_OBJECT)]))

        # macro
        palette.add_block('indexblock',
                          style='basic-style-extended-vertical',
                          label=_('index'),
                          help_string=_('return the text of the positions'))

        palette.add_block('index',
                          hidden=True,
                          style='number-style-var-3arg',
                          label=[_('index') + '\n\n', _('string'),
                                 _('start'), _('end')],
                          prim_name='index',
                          default=[_('text'), 0, 1],
                          string_or_number=True,
                          help_string=_('return the text of the positions'))

        self.tw.lc.def_prim('index', 3,
                            Primitive(self.prim_index,
                                      return_type=TYPE_STRING,
                                      arg_descs=[ArgSlot(TYPE_STRING),
                                                 ArgSlot(TYPE_INT),
                                                 ArgSlot(TYPE_INT)]))

    def _portfolio_palette(self):

        palette = make_palette('portfolio',
                               colors=["#0606FF", "#0606A0"],
                               help_string=_('Palette of presentation \
templates'),
                               position=9,
                               translation=_('portfolio'))

        palette.add_block('hideblocks',
                          style='basic-style-extended-vertical',
                          label=_('hide blocks'),
                          prim_name='hideblocks',
                          help_string=_('declutters canvas by hiding blocks'))
        self.tw.lc.def_prim(
            'hideblocks', 0,
            Primitive(self._prim_hideblocks, export_me=False))

        palette.add_block('showblocks',
                          style='basic-style-extended-vertical',
                          label=_('show blocks'),
                          prim_name='showblocks',
                          help_string=_('restores hidden blocks'))
        self.tw.lc.def_prim(
            'showblocks', 0,
            Primitive(self._prim_showblocks, export_me=False))

        palette.add_block('fullscreen',
                          style='basic-style-extended-vertical',
                          label=_('Fullscreen').lower(),
                          prim_name='fullscreen',
                          help_string=_('hides the Sugar toolbars'))
        self.tw.lc.def_prim(
            'fullscreen', 0,
            Primitive(self.tw.set_fullscreen, export_me=False))

        primitive_dictionary['bulletlist'] = self._prim_list
        palette.add_block('bulletlist',
                          hidden=True,
                          colors=["#0606FF", "#0606A0"],
                          style='bullet-style',
                          label=_('list'),
                          string_or_number=True,
                          prim_name='bulletlist',
                          default=['∙ ', '∙ '],
                          help_string=_('presentation bulleted list'))
        self.tw.lc.def_prim('bulletlist', 1,
                            primitive_dictionary['bulletlist'], True)

        # macros
        palette.add_block('list',
                          style='basic-style-extended',
                          label=' ',
                          help_string=_('presentation template: list of \
bullets'))
        MEDIA_SHAPES.append('list')

        palette.add_block('1x1a',
                          style='basic-style-extended',
                          label=' ',
                          help_string=_('presentation template: select \
Journal object (no description)'))
        MEDIA_SHAPES.append('1x1a')

        palette.add_block('1x1',
                          style='basic-style-extended',
                          label=' ',
                          help_string=_('presentation template: select \
Journal object (with description)'))
        MEDIA_SHAPES.append('1x1')

        palette.add_block('2x2',
                          style='basic-style-extended',
                          label=' ',
                          help_string=_('presentation template: select four \
Journal objects'))
        MEDIA_SHAPES.append('2x2')

        palette.add_block('2x1',
                          style='basic-style-extended',
                          label=' ',
                          help_string=_('presentation template: select two \
Journal objects'))
        MEDIA_SHAPES.append('2x1')

        palette.add_block('1x2',
                          style='basic-style-extended',
                          label=' ',
                          help_string=_('presentation template: select two \
Journal objects'))
        MEDIA_SHAPES.append('1x2')

        # Display-dependent constants
        palette.add_block('leftpos',
                          style='box-style',
                          label=_('left'),
                          prim_name='lpos',
                          logo_command='lpos',
                          help_string=_('xcor of left of screen'))
        self.tw.lc.def_prim(
            'lpos', 0,
            Primitive(CONSTANTS.get, return_type=TYPE_INT,
                      arg_descs=[ConstantArg('leftpos')]))

        palette.add_block('bottompos',
                          style='box-style',
                          label=_('bottom'),
                          prim_name='bpos',
                          logo_command='bpos',
                          help_string=_('ycor of bottom of screen'))
        self.tw.lc.def_prim(
            'bpos', 0,
            Primitive(CONSTANTS.get, return_type=TYPE_INT,
                      arg_descs=[ConstantArg('bottompos')]))

        palette.add_block('width',
                          style='box-style',
                          label=_('width'),
                          prim_name='hres',
                          logo_command='width',
                          help_string=_('the canvas width'))
        self.tw.lc.def_prim(
            'hres', 0,
            Primitive(CONSTANTS.get, return_type=TYPE_INT,
                      arg_descs=[ConstantArg('width')]))

        palette.add_block('rightpos',
                          style='box-style',
                          label=_('right'),
                          prim_name='rpos',
                          logo_command='rpos',
                          help_string=_('xcor of right of screen'))
        self.tw.lc.def_prim(
            'rpos', 0,
            Primitive(CONSTANTS.get, return_type=TYPE_INT,
                      arg_descs=[ConstantArg('rightpos')]))

        palette.add_block('toppos',
                          style='box-style',
                          label=_('top'),
                          prim_name='tpos',
                          logo_command='tpos',
                          help_string=_('ycor of top of screen'))
        self.tw.lc.def_prim(
            'tpos', 0,
            Primitive(CONSTANTS.get, return_type=TYPE_INT,
                      arg_descs=[ConstantArg('toppos')]))

        palette.add_block('height',
                          style='box-style',
                          label=_('height'),
                          prim_name='vres',
                          logo_command='height',
                          help_string=_('the canvas height'))
        self.tw.lc.def_prim(
            'vres', 0,
            Primitive(CONSTANTS.get, return_type=TYPE_INT,
                      arg_descs=[ConstantArg('height')]))

        palette.add_block('titlex',
                          hidden=True,
                          colors=["#0606FF", "#0606A0"],
                          style='box-style',
                          label=_('title x'),
                          logo_command='titlex',
                          prim_name='titlex')
        self.tw.lc.def_prim(
            'titlex', 0,
            Primitive(CONSTANTS.get, return_type=TYPE_INT,
                      arg_descs=[ConstantArg('titlex')]))

        palette.add_block('titley',
                          hidden=True,
                          colors=["#0606FF", "#0606A0"],
                          style='box-style',
                          label=_('title y'),
                          logo_command='titley',
                          prim_name='titley')
        self.tw.lc.def_prim(
            'titley', 0,
            Primitive(CONSTANTS.get, return_type=TYPE_INT,
                      arg_descs=[ConstantArg('titley')]))

        palette.add_block('leftx',
                          hidden=True,
                          colors=["#0606FF", "#0606A0"],
                          style='box-style',
                          label=_('left x'),
                          prim_name='leftx',
                          logo_command='leftx')
        self.tw.lc.def_prim(
            'leftx', 0,
            Primitive(CONSTANTS.get, return_type=TYPE_INT,
                      arg_descs=[ConstantArg('leftx')]))

        palette.add_block('topy',
                          hidden=True,
                          colors=["#0606FF", "#0606A0"],
                          style='box-style',
                          label=_('top y'),
                          prim_name='topy',
                          logo_command='topy')
        self.tw.lc.def_prim(
            'topy', 0,
            Primitive(CONSTANTS.get, return_type=TYPE_INT,
                      arg_descs=[ConstantArg('topy')]))

        palette.add_block('rightx',
                          hidden=True,
                          colors=["#0606FF", "#0606A0"],
                          style='box-style',
                          label=_('right x'),
                          prim_name='rightx',
                          logo_command='rightx')
        self.tw.lc.def_prim(
            'rightx', 0,
            Primitive(CONSTANTS.get, return_type=TYPE_INT,
                      arg_descs=[ConstantArg('rightx')]))

        palette.add_block('bottomy',
                          hidden=True,
                          colors=["#0606FF", "#0606A0"],
                          style='box-style',
                          label=_('bottom y'),
                          prim_name='bottomy',
                          logo_command='bottomy')
        self.tw.lc.def_prim(
            'bottomy', 0,
            Primitive(CONSTANTS.get, return_type=TYPE_INT,
                      arg_descs=[ConstantArg('bottomy')]))

    def _myblocks_palette(self):
        ''' User-defined macros are saved as a json-encoded file;
        these get loaded into a palette on startup '''

        if hasattr(self.tw, 'macros_path') and \
                os.path.exists(self.tw.macros_path):
            files = get_endswith_files(self.tw.macros_path, ".tb")
            if len(files) > 0:
                palette = make_palette(
                    'myblocks',
                    colors=["#FFFF00", "#A0A000"],
                    help_string=_('Palette of user-defined operators'),
                    translation=_('my blocks'))

            for tafile in files:
                data = data_from_file(tafile)
                name = os.path.basename(tafile)[:-3]
                # print 'loading macro %s' % (name)
                MACROS['user-defined-' + name] = hat_on_top(listify(data))
                palette.add_block('user-defined-' + name,
                                  style='basic-style-extended-vertical',
                                  label=name)

    # Block primitives

    def after_keypress(self):
        if self.tw.lc.update_values:
            if self.tw.keypress in KEY_DICT:
                if KEY_DICT[self.tw.keypress] in REVERSE_KEY_DICT:
                    self.tw.lc.update_label_value(
                        'keyboard', REVERSE_KEY_DICT[
                            KEY_DICT[self.tw.keypress]])
                else:
                    self.tw.lc.update_label_value(
                        'keyboard', chr(KEY_DICT[self.tw.keypress]))
            elif self.tw.keyboard > 0:
                self.tw.lc.update_label_value('keyboard',
                                              chr(self.tw.keyboard))
        self.tw.keypress = ''

    def after_pop(self, *ignored_args):
        if self.tw.lc.update_values:
            if not self.tw.lc.heap:
                self.tw.lc.update_label_value('pop')
            else:
                self.tw.lc.update_label_value('pop', self.tw.lc.heap[-1])

    def after_push(self, *ignored_args):
        if self.tw.lc.update_values:
            if not self.tw.lc.heap:
                self.tw.lc.update_label_value('pop')
            else:
                self.tw.lc.update_label_value('pop', self.tw.lc.heap[-1])

    def prim_speak(self, text):
        """ Speak text """
        if isinstance(text, float) and int(text) == text:
            text = int(text)

        lang = os.environ['LANG'][0:2]
        if lang in VOICES:
            language_option = '-v ' + VOICES[lang]
        else:
            language_option = ''

        if self.tw.running_sugar:
            from sugar3.speech import SpeechManager
            sm = SpeechManager()
            sm.say_text(text)
        else:
            os.system(
                'espeak %s "%s" --stdout | aplay' %
                (language_option, str(text)))

        if self.tw.sharing():
            if language_option == '':
                event = 'S|%s' % (data_to_string([self.tw.nick, 'None', text]))
            else:
                event = 'S|%s' % (data_to_string([self.tw.nick,
                                                  language_option, text]))
            self.tw.send_event(event)

    def prim_sinewave(self, pitch, amplitude, duration):
        """ Create a Csound score to play a sine wave. """
        self.orchlines = []
        self.scorelines = []
        self.instrlist = []

        try:
            pitch = abs(float(pitch))
            amplitude = abs(float(amplitude))
            duration = abs(float(duration))
        except ValueError:
            self.tw.lc.stop_logo()
            raise logoerror("#notanumber")

        self._play_sinewave(pitch, amplitude, duration)

        if self.tw.running_sugar:
            path = os.path.join(get_path(self.tw.activity, 'instance'),
                                'tmp.csd')
        else:
            path = os.path.join(tempfile.gettempdir(), 'tmp.csd')
        # Create a csound file from the score.
        self._audio_write(path)
        # Play the csound file.
        os.system('csound ' + path + ' > /dev/null 2>&1')

    def _play_sinewave(self, pitch, amplitude, duration, starttime=0,
                       pitch_envelope=99, amplitude_envelope=100,
                       instrument=1):

        pitenv = pitch_envelope
        ampenv = amplitude_envelope
        if 1 not in self.instrlist:
            self.orchlines.append("instr 1\n")
            self.orchlines.append("kpitenv oscil 1, 1/p3, p6\n")
            self.orchlines.append("aenv oscil 1, 1/p3, p7\n")
            self.orchlines.append("asig oscil p5*aenv, p4*kpitenv, p8\n")
            self.orchlines.append("out asig\n")
            self.orchlines.append("endin\n\n")
            self.instrlist.append(1)

        self.scorelines.append("i1 %s %s %s %s %s %s %s\n" %
                               (str(starttime), str(duration), str(pitch),
                                str(amplitude), str(pitenv), str(ampenv),
                                str(instrument)))

    def _audio_write(self, file):
        """ Compile a .csd file. """

        csd = open(file, "w")
        csd.write("<CsoundSynthesizer>\n\n")
        csd.write("<CsOptions>\n")
        csd.write("-+rtaudio=alsa -odevaudio -m0 -d -b256 -B512\n")
        csd.write("</CsOptions>\n\n")
        csd.write("<CsInstruments>\n\n")
        csd.write("sr=16000\n")
        csd.write("ksmps=50\n")
        csd.write("nchnls=1\n\n")
        for line in self.orchlines:
            csd.write(line)
        csd.write("\n</CsInstruments>\n\n")
        csd.write("<CsScore>\n\n")
        csd.write("f1 0 2048 10 1\n")
        csd.write("f2 0 2048 10 1 0 .33 0 .2 0 .143 0 .111\n")
        csd.write("f3 0 2048 10 1 .5 .33 .25 .2 .175 .143 .125 .111 .1\n")
        csd.write("f10 0 2048 10 1 0 0 .3 0 .2 0 0 .1\n")
        csd.write("f99 0 2048 7 1 2048 1\n")
        csd.write("f100 0 2048 7 0. 10 1. 1900 1. 132 0.\n")
        csd.write(self.scorelines.pop())
        csd.write("e\n")
        csd.write("\n</CsScore>\n")
        csd.write("\n</CsoundSynthesizer>")
        csd.close()

    def after_mouse_x(self):
        """ Show mouse x coordinate """
        if self.tw.lc.update_values:
            self.tw.lc.update_label_value('mousex', self.tw.get_mouse_x())

    def after_mouse_y(self):
        """ Show mouse y coordinate """
        if self.tw.lc.update_values:
            self.tw.lc.update_label_value('mousey', self.tw.get_mouse_y())

    def after_see(self):
        """ Show color under turtle """
        if self.tw.lc.update_values:
            self.tw.lc.update_label_value(
                'see',
                self.tw.turtles.get_active_turtle().get_color_index())

    def _prim_list(self, blklist):
        """ Expandable list block """
        self.tw.lc.showlist(blklist)
        self.tw.lc.ireturn()
        yield True

    def after_time(self, elapsed_time):
        """ Update the label of the 'time' block after computing the new
        value. """
        if self.tw.lc.update_values:
            self.tw.lc.update_label_value('time', elapsed_time)

    def _prim_hideblocks(self):
        """ hide blocks and show showblocks button """
        self.tw.hideblocks()
        self.tw.lc.trace = 0
        self.tw.step_time = 0
        if self.tw.running_sugar:
            self.tw.activity.stop_turtle_button.set_icon_name("hideshowoff")
            self.tw.activity.stop_turtle_button.set_tooltip(_('Show blocks'))

    def _prim_showblocks(self):
        """ show blocks and show stop turtle button """
        self.tw.showblocks()
        self.tw.lc.trace = 1
        self.tw.step_time = 3
        if self.tw.running_sugar:
            self.tw.activity.stop_turtle_button.set_icon_name("stopiton")
            self.tw.activity.stop_turtle_button.set_tooltip(_('Stop turtle'))

    def after_set(self, name, value=None):
        ''' Update the associated value blocks '''
        if value is not None:
            if self.tw.lc.update_values:
                self.tw.lc.update_label_value(name, value)

    def prim_index(self, string, start, end):
        start = int(start)
        end = int(end)

        return string[start:end]



from device import RFIDDevice
from serial import Serial
import dbus
from dbus.mainloop.glib import DBusGMainLoop
import gobject
from time import sleep
import utils

HAL_SERVICE = 'org.freedesktop.Hal'
HAL_MGR_PATH = '/org/freedesktop/Hal/Manager'
HAL_MGR_IFACE = 'org.freedesktop.Hal.Manager'
HAL_DEV_IFACE = 'org.freedesktop.Hal.Device'
REGEXP_SERUSB = '\/org\/freedesktop\/Hal\/devices\/usb_device['\
                'a-z,A-Z,0-9,_]*serial_usb_[0-9]'

VERSIONS = ['301']


class RFIDReader(RFIDDevice):
    """
    RFIDRW-E-W interface.
    """

    def __init__(self):

        RFIDDevice.__init__(self)
        self.last_tag = ""
        self.tags = []
        self.ser = Serial()
        self.device = ''
        self.device_path = ''
        self._connected = False

        loop = DBusGMainLoop()
        self.bus = dbus.SystemBus(mainloop=loop)
        hmgr_iface = dbus.Interface(
            self.bus.get_object(
                HAL_SERVICE,
                HAL_MGR_PATH),
            HAL_MGR_IFACE)

        hmgr_iface.connect_to_signal('DeviceRemoved', self._device_removed_cb)

    def get_present(self):
        """
        Checks if RFID-RW-USB device is present.
        Returns True if so, False otherwise.
        """
        hmgr_if = dbus.Interface(
            self.bus.get_object(
                HAL_SERVICE,
                HAL_MGR_PATH),
            HAL_MGR_IFACE)
        serialusb_devices = set(
            hmgr_if.FindDeviceStringMatch(
                'serial.type', 'usb')) & set(
            hmgr_if.FindDeviceStringMatch(
                'info.subsystem', 'tty'))
        for i in serialusb_devices:
            serialusb_if = dbus.Interface(self.bus.get_object(HAL_SERVICE, i),
                                          HAL_DEV_IFACE)
            if serialusb_if.PropertyExists('info.parent'):
                parent_udi = str(serialusb_if.GetProperty('info.parent'))
                parent = dbus.Interface(
                    self.bus.get_object(
                        HAL_SERVICE,
                        parent_udi),
                    HAL_DEV_IFACE)
                if parent.PropertyExists('info.linux.driver') and str(
                        parent.GetProperty('info.linux.driver')) == 'ftdi_sio':
                    device = str(serialusb_if.GetProperty('linux.device_file'))
                    ser = Serial(device, 9600, timeout=0.1)
                    ser.read(100)
                    ser.write('v')
                    ser.write('e')
                    ser.write('r')
                    ser.write('\x0D')
                    resp = ser.read(4)
                    if resp[0:-1] in VERSIONS:
                        self.device = device
                        self.device_path = i
                        return True
        return False

    def do_connect(self):
        """
        Connects to the device.
        Returns True if successfull, False otherwise.
        """
        retval = False
        if self.get_present():
            try:
                self.ser = Serial(self.device, 9600, timeout=0.1)
                self._connected = True
                if self._select_animal_tag:
                    # gobject.idle_add(self._loop)
                    gobject.timeout_add(1000, self._loop)
                    retval = True
            except BaseException:
                self._connected = False
        return retval

    def do_disconnect(self):
        """
        Disconnect from the device.
        """
        self.ser.close()
        self._connected = False

    def read_tag(self):
        """
        Returns the last read value.
        """
        return self.last_tag

    def _select_animal_tag(self):
        """
        Sends the "Select Tag 2" (animal tag) command to the device.
        """
        self.ser.read(100)
        self.ser.write('s')
        self.ser.write('t')
        self.ser.write('2')
        self.ser.write('\x0d')
        resp = self.ser.read(3)[0:-1]
        if resp == 'OK':
            return True
        return False

    def get_version(self):
        """
        Sends the version command to the device and returns
        a string with the device version.
        """
        # self.ser.flushInput()
        ver = "???"
        self.ser.read(100)
        self.ser.write('v')
        self.ser.write('e')
        self.ser.write('r')
        self.ser.write('\x0d')
        resp = self.ser.read(4)[0:-1]
        if resp in VERSIONS:
            return "RFIDRW-E-USB " + resp
        return ver

    def _device_removed_cb(self, path):
        """
        Called when a device is removed.
        Checks if the removed device is itself and emits the "disconnected"
        signal if so.
        """
        if path == self.device_path:
            self.device_path = ''
            self.ser.close()
            self._connected = False
            self.tags = []
            self.emit("disconnected", "RFID-RW-USB")

    def _loop(self):
        """
        Threaded loop for reading data from the device.
        """
        if not self._connected:
            return False

        self.ser.read(100)
        self.ser.write('r')
        self.ser.write('a')
        self.ser.write('t')
        self.ser.write('\x0d')
        resp = self.ser.read(33)[0:-1].split('_')
        if resp.__len__() is not 6 or resp in self.tags:
            return True

        self.tags.append(resp)
        anbit_bin = utils.dec2bin(int(resp[2]))
        reserved_bin = '00000000000000'
        databit_bin = utils.dec2bin(int(resp[3]))
        country_bin = utils.dec2bin(int(resp[0]))
        while country_bin.__len__() < 10:
            country_bin = '0' + country_bin
        id_bin = utils.dec2bin(int(resp[1]))
        while id_bin.__len__() < 10:
            id_bin = '0' + id_bin

        tag_bin = anbit_bin + reserved_bin + databit_bin + country_bin + id_bin
        data = utils.bin2hex(tag_bin)
        self.emit("tag-read", data)
        self.last_tag = data
        # sleep(1)
        return True

# Testing
# if __name__ == '__main__':
#    def handler(device, idhex):
#        """
#        Handler for "tag-read" signal.
#        Prints the tag id.
#        """
#        print "ID: ", idhex
#
#    dev = RFIDReader()
#    if dev.get_present():
#        print "SIPI!"
#        dev.do_connect()
#        dev.connect('tag-read', handler)
#    else:
#        print "Not connected"
#
#    mloop = gobject.MainLoop()
#    mloop.run()



# Copyright (c) 2014, Walter Bender

# Usage: Import this code into a Python (user-definable) block; when
# this code is run, the turtle will draw a line composed of text of
# the length of the numeric argument block docked to the Python block.
# A second, optional, argument can be used to specify the text. The
# default is '*'.


def myblock(tw, args):
    ''' Draw a line of length line_length composed of text characters '''

    try:  # make sure line_length is a number
        line_length = float(args[0])
    except ValueError:
        return
    # An optional second argument can be used for setting the text.
    if len(args) > 1:
        string = str(args[1])
    else:
        string = '*'

    turtle = tw.turtles.get_active_turtle()
    if turtle.get_pen_state():
        dist = 0
        pen_size = turtle.get_pen_size()
        heading = turtle.get_heading()
        i = 0  # index into text string
        while dist + pen_size < line_length:
            turtle.set_pen_state(True)
            x, y = tw.turtles.turtle_to_screen_coordinates(
                turtle.get_xy())
            turtle.set_heading(heading - 90)
            turtle.draw_text(string[i % len(string)], x, y, pen_size,
                             pen_size * 1.5)
            i += 1
            turtle.set_heading(heading)
            turtle.set_pen_state(False)
            turtle.forward(pen_size * 1.5)
            dist += pen_size * 1.5
        # make sure we have moved exactly line_length
        turtle.forward(line_length - dist)
        turtle.set_pen_state(True)
    else:
        turtle.forward(line_length)



# Copyright (c) 2007-8, Playful Invention Company.
# Copyright (c) 2008-11, Walter Bender
# Copyright (c) 2011 Collabora Ltd. <http://www.collabora.co.uk/>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import cairo
import os

from math import pi

from gi.repository import Gdk
from gi.repository import Pango
from gi.repository import PangoCairo
from .tautils import get_path
from .taconstants import (Color, TMP_SVG_PATH, DEFAULT_PEN_COLOR,
                         DEFAULT_BACKGROUND_COLOR, DEFAULT_FONT)


def wrap100(n):
    ''' A variant on mod... 101 -> 99; 199 -> 1 '''
    n = int(n)
    n %= 200
    if n > 99:
        n = 199 - n
    return n


def calc_shade(c, s, invert=False):
    ''' Convert a color to the current shade (lightness/darkness). '''
    # Assumes 16 bit input values
    if invert:
        if s == -1:
            return int(c)
        elif s < 0:
            return int(c / (1 + s))
        return int((c - 65536 * s) / (1 - s))
    else:
        if s < 0:
            return int(c * (1 + s))
        return int(c + (65536 - c) * s)


def calc_gray(c, g, invert=False):
    ''' Gray is a psuedo saturation calculation. '''
    # Assumes 16 bit input values
    if g == 100:
        return int(c)
    if invert:
        if g == 0:
            return int(c)
        else:
            return int(((c * 100) - (32768 * (100 - g))) / g)
    else:
        return int(((c * g) + (32768 * (100 - g))) / 100)


colors = {}
DEGTOR = pi / 180.
RTODEG = 180. / pi

COLOR_TABLE = (
    0xFF0000, 0xFF0D00, 0xFF1A00, 0xFF2600, 0xFF3300,
    0xFF4000, 0xFF4D00, 0xFF5900, 0xFF6600, 0xFF7300,
    0xFF8000, 0xFF8C00, 0xFF9900, 0xFFA600, 0xFFB300,
    0xFFBF00, 0xFFCC00, 0xFFD900, 0xFFE600, 0xFFF200,
    0xFFFF00, 0xE6FF00, 0xCCFF00, 0xB3FF00, 0x99FF00,
    0x80FF00, 0x66FF00, 0x4DFF00, 0x33FF00, 0x1AFF00,
    0x00FF00, 0x00FF0D, 0x00FF1A, 0x00FF26, 0x00FF33,
    0x00FF40, 0x00FF4D, 0x00FF59, 0x00FF66, 0x00FF73,
    0x00FF80, 0x00FF8C, 0x00FF99, 0x00FFA6, 0x00FFB3,
    0x00FFBF, 0x00FFCC, 0x00FFD9, 0x00FFE6, 0x00FFF2,
    0x00FFFF, 0x00F2FF, 0x00E6FF, 0x00D9FF, 0x00CCFF,
    0x00BFFF, 0x00B3FF, 0x00A6FF, 0x0099FF, 0x008CFF,
    0x0080FF, 0x0073FF, 0x0066FF, 0x0059FF, 0x004DFF,
    0x0040FF, 0x0033FF, 0x0026FF, 0x001AFF, 0x000DFF,
    0x0000FF, 0x0D00FF, 0x1A00FF, 0x2600FF, 0x3300FF,
    0x4000FF, 0x4D00FF, 0x5900FF, 0x6600FF, 0x7300FF,
    0x8000FF, 0x8C00FF, 0x9900FF, 0xA600FF, 0xB300FF,
    0xBF00FF, 0xCC00FF, 0xD900FF, 0xE600FF, 0xF200FF,
    0xFF00FF, 0xFF00E6, 0xFF00CC, 0xFF00B3, 0xFF0099,
    0xFF0080, 0xFF0066, 0xFF004D, 0xFF0033, 0xFF001A)


class TurtleGraphics:

    ''' A class for the Turtle graphics canvas '''

    def __init__(self, turtle_window, width, height):
        ''' Create a sprite to hold the canvas. '''
        self.turtle_window = turtle_window
        self.width = width
        self.height = height
        self.textsize = 48
        self._fgrgb = DEFAULT_PEN_COLOR
        self._bgrgb = DEFAULT_BACKGROUND_COLOR
        self._font = DEFAULT_FONT
        self._shade = 0
        self._color = 0
        self._gray = 100
        self.cr_svg = None  # Surface used for saving to SVG

        # Build a cairo.Context from a cairo.XlibSurface
        self.canvas = cairo.Context(self.turtle_window.turtle_canvas)
        self.set_pen_size(5)

    def setup_svg_surface(self):
        ''' Set up a surface for saving to SVG '''
        svg_surface = cairo.SVGSurface(self.get_svg_path(),
                                       self.width, self.height)
        self.svg_surface = svg_surface
        self.cr_svg = cairo.Context(svg_surface)
        self.cr_svg.set_line_cap(1)  # Set the line cap to be round

    def get_svg_path(self):
        '''We use a separate file for the svg used for generating Sugar icons
        '''
        if self.turtle_window.running_sugar:
            return os.path.join(get_path(self.turtle_window.activity,
                                         'instance'), 'output.svg')
        else:
            return TMP_SVG_PATH

    def fill_polygon(self, poly_points):
        ''' Draw the polygon... '''
        def _fill_polygon(cr, poly_points):
            cr.new_path()
            for i, p in enumerate(poly_points):
                if p[0] == 'move':
                    if i == len(poly_points) - 1 or \
                       poly_points[i + 1][0] not in ['rarc', 'larc']:
                        cr.move_to(p[1], p[2])
                elif p[0] == 'rarc':
                    cr.arc(p[1], p[2], p[3], p[4], p[5])
                elif p[0] == 'larc':
                    cr.arc_negative(p[1], p[2], p[3], p[4], p[5])
                else:  # line
                    cr.line_to(p[1], p[2])
            cr.close_path()
            cr.fill()

        _fill_polygon(self.canvas, poly_points)
        self.inval()
        if self.cr_svg is not None:
            _fill_polygon(self.cr_svg, poly_points)

    def clearscreen(self):
        '''Clear the canvas and reset most graphics attributes to defaults.'''

        def _clearscreen(cr):
            cr.move_to(0, 0)
            self._bgrgb = DEFAULT_BACKGROUND_COLOR
            cr.set_source_rgb(self._bgrgb[0] / 255.,
                              self._bgrgb[1] / 255.,
                              self._bgrgb[2] / 255.)
            cr.rectangle(0, 0, self.width * 2, self.height * 2)
            cr.fill()

        _clearscreen(self.canvas)
        self.inval()
        if self.cr_svg is not None:
            _clearscreen(self.cr_svg)

    def rarc(self, x, y, r, a, heading):
        ''' draw a clockwise arc '''
        def _rarc(cr, x, y, r, a, h):
            cr.arc(x, y, r, (h - 180) * DEGTOR, (h - 180 + a) * DEGTOR)
            cr.stroke()

        _rarc(self.canvas, x, y, r, a, heading)
        self.inval()

        if self.cr_svg is not None:
            _rarc(self.cr_svg, x, y, r, a, heading)

    def larc(self, x, y, r, a, heading):
        ''' draw a counter-clockwise arc '''
        def _larc(cr, x, y, r, a, h):
            cr.arc_negative(x, y, r, h * DEGTOR, (h - a) * DEGTOR)
            cr.stroke()

        _larc(self.canvas, x, y, r, a, heading)
        self.inval()
        if self.cr_svg is not None:
            _larc(self.cr_svg, x, y, r, a, heading)

    def set_pen_size(self, pen_size):
        ''' Set the pen size '''
        self.canvas.set_line_width(pen_size)
        if self.cr_svg is not None:
            self.cr_svg.set_line_width(pen_size)

    def fillscreen(self, c, s):
        ''' Deprecated method: Fill screen with color/shade '''
        self.fillscreen_with_gray(c, s, self._gray)

    def fillscreen_with_gray(self, color, shade, gray):
        ''' Fill screen with color/shade/gray and reset to defaults '''

        save_rgb = self._fgrgb[:]

        # Special case for color blocks
        if isinstance(color, Color):
            if color.color is None:
                self._shade = color.shade
            else:
                self._color = color.color
        else:
            self._color = color
        if isinstance(shade, Color):
            self._shade = shade.shade
        else:
            self._shade = shade
        if isinstance(gray, Color):
            self._gray = gray.gray
        else:
            self._gray = gray

        if self._gray < 0:
            self._gray = 0
        if self._gray > 100:
            self._gray = 100

        self.set_fgcolor(shade=self._shade, gray=self._gray, color=self._color)
        self._bgrgb = self._fgrgb[:]

        def _fillscreen(cr, rgb, w, h):
            cr.set_source_rgb(rgb[0] / 255., rgb[1] / 255., rgb[2] / 255.)
            cr.rectangle(0, 0, w * 2, h * 2)
            cr.fill()

        _fillscreen(self.canvas, self._fgrgb, self.width, self.height)
        self.inval()
        if self.cr_svg is not None:
            _fillscreen(self.cr_svg, self._fgrgb, self.width, self.height)

        self._fgrgb = save_rgb[:]

    def set_fgcolor(self, shade=None, gray=None, color=None):
        ''' Set the foreground color '''
        if shade is not None:
            self._shade = shade
        if gray is not None:
            self._gray = gray
        if color is not None:
            self._color = color
        sh = (wrap100(self._shade) - 50) / 50.0
        rgb = COLOR_TABLE[wrap100(self._color)]
        r = (rgb >> 8) & 0xff00
        r = calc_gray(r, self._gray)
        r = calc_shade(r, sh)
        g = rgb & 0xff00
        g = calc_gray(g, self._gray)
        g = calc_shade(g, sh)
        b = (rgb << 8) & 0xff00
        b = calc_gray(b, self._gray)
        b = calc_shade(b, sh)
        self._fgrgb = [r >> 8, g >> 8, b >> 8]

    def draw_surface(self, surface, x, y, w, h):
        ''' Draw a surface '''

        def _draw_surface(cc, surface, x, y, w, h):
            cc.set_source_surface(surface, x, y)
            cc.rectangle(x, y, w, h)
            cc.fill()

        _draw_surface(self.canvas, surface, x, y, w, h)
        self.inval()
        if self.cr_svg is not None:
            _draw_surface(self.cr_svg, surface, x, y, w, h)

    def draw_pixbuf(self, pixbuf, a, b, x, y, w, h, heading):
        ''' Draw a pixbuf '''

        def _draw_pixbuf(cc, pixbuf, a, b, x, y, w, h, heading):
            # Build a gtk.gdk.CairoContext from a cairo.Context to access
            # the set_source_pixbuf attribute.
            cc.save()
            # center the rotation on the center of the image
            cc.translate(x + w / 2., y + h / 2.)
            cc.rotate(heading * DEGTOR)
            cc.translate(-x - w / 2., -y - h / 2.)
            Gdk.cairo_set_source_pixbuf(cc, pixbuf, x, y)
            cc.rectangle(x, y, w, h)
            cc.fill()
            cc.restore()

        _draw_pixbuf(self.canvas, pixbuf, a, b, x, y, w, h, heading)
        self.inval()
        if self.cr_svg is not None:
            _draw_pixbuf(self.cr_svg, pixbuf, a, b, x, y, w, h, heading)

    def set_font(self, font_name):
        ''' Set font used by draw_text '''
        self._font = str(font_name)

    def draw_text(self, label, x, y, size, width, heading, scale):
        ''' Draw text '''

        def _draw_text(cc, label, x, y, size, width, scale, heading, rgb,
                       wrap=False):
            import textwrap

            final_scale = int(size * scale) * Pango.SCALE
            label = str(label)
            if wrap:
                label = '\n'.join(textwrap.wrap(label, int(width / scale)))

            pl = PangoCairo.create_layout(cc)
            fd = Pango.FontDescription(self._font)
            fd.set_size(final_scale)
            pl.set_font_description(fd)
            if isinstance(label, str):
                text = label.replace('\0', ' ')
            elif isinstance(label, (float, int)):
                text = str(label)
            else:
                text = label

            pl.set_text(text, -1)
            pl.set_width(int(width) * Pango.SCALE)
            cc.save()
            cc.translate(x, y)
            cc.rotate(heading * DEGTOR)
            cc.set_source_rgb(rgb[0] / 255., rgb[1] / 255., rgb[2] / 255.)
            PangoCairo.update_layout(cc, pl)
            PangoCairo.show_layout(cc, pl)
            cc.restore()

        width *= scale
        _draw_text(self.canvas, label, x, y, size, width, scale, heading,
                   self._fgrgb)
        self.inval()
        if self.cr_svg is not None:  # and self.pendown:
            _draw_text(self.cr_svg, label, x, y, size, width, scale, heading,
                       self._fgrgb, wrap=True)

    def set_source_rgb(self):
        r = self._fgrgb[0] / 255.
        g = self._fgrgb[1] / 255.
        b = self._fgrgb[2] / 255.
        self.canvas.set_source_rgb(r, g, b)
        if self.cr_svg is not None:
            self.cr_svg.set_source_rgb(r, g, b)

    def draw_line(self, x1, y1, x2, y2):
        ''' Draw a line '''

        def _draw_line(cr, x1, y1, x2, y2):
            cr.set_line_cap(1)  # Set the line cap to be round
            cr.move_to(x1, y1)
            cr.line_to(x2, y2)
            cr.stroke()

        _draw_line(self.canvas, x1, y1, x2, y2)
        if self.cr_svg is not None:
            _draw_line(self.cr_svg, x1, y1, x2, y2)
        self.inval()

    def get_color_index(self, r, g, b, a=0):
        ''' Find the closest palette entry to the rgb triplet '''
        if self._shade != 50 or self._gray != 100:
            r <<= 8
            g <<= 8
            b <<= 8
            if self._shade != 50:
                sh = (wrap100(self._shade) - 50) / 50.
                r = calc_shade(r, sh, True)
                g = calc_shade(g, sh, True)
                b = calc_shade(b, sh, True)
            if self._gray != 100:
                r = calc_gray(r, self._gray, True)
                g = calc_gray(g, self._gray, True)
                b = calc_gray(b, self._gray, True)
            r >>= 8
            g >>= 8
            b >>= 8
        min_distance = 1000000
        closest_color = -1
        for i, c in enumerate(COLOR_TABLE):
            cr = int((c & 0xff0000) >> 16)
            cg = int((c & 0x00ff00) >> 8)
            cb = int((c & 0x0000ff))
            distance_squared = \
                ((cr - r) ** 2) + ((cg - g) ** 2) + ((cb - b) ** 2)
            if distance_squared == 0:
                return i
            if distance_squared < min_distance:
                min_distance = distance_squared
                closest_color = i
        return closest_color

    def get_pixel(self, x, y):
        ''' Read the pixel at x, y '''
        if self.turtle_window.interactive_mode:
            x = int(x)
            y = int(y)
            w = self.turtle_window.turtle_canvas.get_width()
            h = self.turtle_window.turtle_canvas.get_height()
            if x < 0 or x > (w - 1) or y < 0 or y > (h - 1):
                return(-1, -1, -1, -1)
            # create a new 1x1 cairo surface
            cs = cairo.ImageSurface(cairo.FORMAT_RGB24, 1, 1)
            cr = cairo.Context(cs)
            cr.set_source_surface(self.turtle_window.turtle_canvas, -x, -y)
            cr.rectangle(0, 0, 1, 1)
            cr.set_operator(cairo.OPERATOR_SOURCE)
            cr.fill()
            cs.flush()  # ensure all writing is done
            pixels = cs.get_data()  # Read the pixel
            return (pixels[2], pixels[1], pixels[0], 0)
        else:
            return(-1, -1, -1, -1)

    def svg_close(self):
        ''' Close current SVG graphic '''
        self.cr_svg.show_page()
        self.svg_surface.flush()
        self.svg_surface.finish()

    def svg_reset(self):
        ''' Reset svg flags '''
        self.cr_svg = None

    def inval(self):
        ''' Invalidate a region for gtk '''
        self.turtle_window.inval_all()



"""
 tagplay.py
 refactored based on Jukebox Activity
 Copyright (C) 2007 Andy Wingo <wingo@pobox.com>
 Copyright (C) 2007 Red Hat, Inc.
 Copyright (C) 2008-2010 Kushal Das <kushal@fedoraproject.org>
 Copyright (C) 2010 Walter Bender
"""

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

import os

from gi.repository import Gtk
from gi.repository import Gst
from gi.repository import Gdk
from gi.repository import GObject
from gi.repository import GdkX11, GstVideo  # noqa: F401
GObject.threads_init()
Gst.init(None)


from .tautils import error_output, debug_output


def play_audio_from_file(lc, file_path):
    """ Called from Show block of audio media """
    if lc.gplay is not None and lc.gplay.player is not None:
        if lc.gplay.player.playing:
            lc.gplay.player.stop()
        if lc.gplay.bin is not None:
            lc.gplay.bin.destroy()

    lc.gplay = Gplay(lc, lc.tw.canvas.width, lc.tw.canvas.height, 4, 3)
    lc.gplay.start(file_path)


def play_movie_from_file(lc, filepath, x, y, w, h):
    """ Called from Show block of video media """
    if lc.gplay is not None and lc.gplay.player is not None:
        if lc.gplay.player.playing:
            lc.gplay.player.stop()
        if lc.gplay.bin is not None:
            lc.gplay.bin.destroy()

    lc.gplay = Gplay(lc, x, y, w, h)
    lc.gplay.start(filepath)


def stop_media(lc):
    """ Called from Clean block and toolbar Stop button """
    if lc.gplay is None:
        return False

    if lc.gplay.player is not None:
        lc.gplay.player.stop()
    if lc.gplay.bin is not None:
        lc.gplay.bin.destroy()

    lc.gplay = None


def pause_media(lc):
    """ From pause media block """
    if lc.gplay is None:
        return False

    if lc.gplay.player is not None:
        lc.gplay.player.pause()


def play_media(lc):
    """ From play media block """
    if lc.gplay is None:
        return False

    if lc.gplay.player is not None:
        lc.gplay.player.play()


def media_playing(lc):
    if lc.gplay is None:
        return False
    return lc.gplay.player.is_playing()


class Gplay():
    UPDATE_INTERVAL = 500

    def __init__(self, lc, x, y, w, h):
        self.running_sugar = lc.tw.running_sugar
        self.player = None
        self.uri = None
        self.playlist = []
        self.jobjectlist = []
        self.playpath = None
        self.only_audio = False
        self.got_stream_info = False
        self.currentplaying = 0

        self.bin = Gtk.Window()

        self.videowidget = VideoWidget()
        self.bin.add(self.videowidget)
        self.bin.set_type_hint(Gdk.WindowTypeHint.NORMAL)
        self.bin.set_decorated(False)
        if self.running_sugar:
            self.bin.set_transient_for(lc.tw.activity)

        self.bin.move(x, y)
        self.bin.resize(w, h)
        self.bin.show_all()

        self._want_document = True

    def _player_eos_cb(self, widget):
        debug_output('end of stream', self.running_sugar)
        # Make sure player is stopped after EOS
        self.player.stop()

    def _player_error_cb(self, widget, message, detail):
        self.player.stop()
        self.player.set_uri(None)
        error_output('Error: %s - %s' % (message, detail),
                     self.running_sugar)

    def _player_stream_info_cb(self, widget, stream_info):
        if not len(stream_info) or self.got_stream_info:
            return

        GST_STREAM_TYPE_VIDEO = 2

        only_audio = True
        for item in stream_info:
            if item.props.type == GST_STREAM_TYPE_VIDEO:
                only_audio = False
        self.only_audio = only_audio
        self.got_stream_info = True

    def start(self, file_path=None):
        self._want_document = False
        self.playpath = os.path.dirname(file_path)
        if not file_path:
            return False
        self.playlist.append('file://' + os.path.abspath(file_path))
        if not self.player:
            # Lazy init the player so that videowidget is realized
            # and has a valid widget allocation.
            self.player = GstPlayer(self.videowidget, self.running_sugar)
            self.player.connect('eos', self._player_eos_cb)
            self.player.connect('error', self._player_error_cb)
            self.player.connect('stream-info', self._player_stream_info_cb)

        try:
            if not self.currentplaying:
                debug_output('Playing: %s' % (self.playlist[0]),
                             self.running_sugar)
                self.player.set_uri(self.playlist[0])
                self.currentplaying = 0
                self.play_toggled()
        except Exception as e:
            error_output('Error playing %s: %s' % (self.playlist[0], e),
                         self.running_sugar)
        return False

    def play_toggled(self):
        if self.player.is_playing():
            self.player.pause()
        else:
            if self.player.error:
                pass
            else:
                self.player.play()


class GstPlayer(GObject.GObject):
    __gsignals__ = {
        'error': (GObject.SIGNAL_RUN_FIRST, None, [str, str]),
        'eos': (GObject.SIGNAL_RUN_FIRST, None, []),
        'stream-info': (GObject.SIGNAL_RUN_FIRST, None, [object])}

    def __init__(self, videowidget, running_sugar):
        GObject.GObject.__init__(self)

        self.running_sugar = running_sugar
        self.playing = False
        self.error = False

        self.player = Gst.ElementFactory.make('playbin', 'player')

        videowidget.realize()
        self.videowidget = videowidget
        self.videowidget_xid = videowidget.get_property('window').get_xid()
        self._init_video_sink()

        bus = self.player.get_bus()
        bus.enable_sync_message_emission()
        bus.add_signal_watch()
        bus.connect('sync-message::element', self.on_sync_message)
        bus.connect('message', self.on_message)

    def set_uri(self, uri):
        self.player.set_property('uri', uri)

    def on_sync_message(self, bus, message):
        if message.get_structure() is None:
            return
        if message.get_structure().get_name() == 'prepare-window-handle':
            self.videowidget.set_sink(message.src, self.videowidget_xid)
            message.src.set_property('force-aspect-ratio', True)

    def on_message(self, bus, message):
        t = message.type
        if t == Gst.MessageType.ERROR:
            err, debug = message.parse_error()
            error_output('Error: %s - %s' % (err, debug),
                         self.running_sugar)
            self.error = True
            self.emit('eos')
            self.playing = False
            self.emit('error', str(err), str(debug))
        elif t == Gst.MessageType.EOS:
            self.emit('eos')
            self.playing = False
        elif t == Gst.MessageType.STATE_CHANGED:
            old, new, pen = message.parse_state_changed()
            if old == Gst.State.READY and new == Gst.State.PAUSED and \
               hasattr(self.player.props, 'stream_info_value_array'):
                self.emit('stream-info',
                          self.player.props.stream_info_value_array)

    def _init_video_sink(self):
        self.bin = Gst.Bin.new()
        videoscale = Gst.ElementFactory.make('videoscale', None)
        self.bin.add(videoscale)
        pad = videoscale.get_static_pad('sink')
        ghostpad = Gst.GhostPad.new('sink', pad)
        self.bin.add_pad(ghostpad)
        videoscale.set_property('method', 0)

        caps_string = 'video/x-raw, '
        r = self.videowidget.get_allocation()
        if r.width > 500 and r.height > 500:
            # Sigh... xvimagesink on the XOs will scale the video to fit
            # but ximagesink in Xephyr does not.  So we live with unscaled
            # video in Xephyr so that the XO can work right.
            w = 480
            h = float(w) / float(float(r.width) / float(r.height))
            caps_string += 'width=%d, height=%d' % (w, h)
        else:
            caps_string += 'width=480, height=360'
        caps = Gst.Caps.from_string(caps_string)
        self.filter = Gst.ElementFactory.make('capsfilter', 'filter')
        self.bin.add(self.filter)
        self.filter.set_property('caps', caps)

        conv = Gst.ElementFactory.make('videoconvert', 'conv')
        self.bin.add(conv)
        videosink = Gst.ElementFactory.make('autovideosink')
        self.bin.add(videosink)

        videoscale.link(self.filter)
        self.filter.link(conv)
        conv.link(videosink)

        self.player.set_property('video-sink', self.bin)

    def pause(self):
        self.player.set_state(Gst.State.PAUSED)
        self.playing = False
        # debug_output('pausing player', self.running_sugar)

    def play(self):
        self.player.set_state(Gst.State.PLAYING)
        self.playing = True
        self.error = False
        # debug_output('playing player', self.running_sugar)

    def stop(self):
        self.player.set_state(Gst.State.NULL)
        self.playing = False
        # debug_output('stopped player', self.running_sugar)

    def get_state(self, timeout=1):
        return self.player.get_state(timeout=timeout)

    def is_playing(self):
        return self.playing


class VideoWidget(Gtk.DrawingArea):

    def __init__(self):
        Gtk.DrawingArea.__init__(self)
        self.set_events(Gdk.EventMask.EXPOSURE_MASK)
        self.imagesink = None
        self.set_double_buffered(True)
        self.set_app_paintable(True)

    def do_expose_event(self, event):
        if self.imagesink:
            self.imagesink.expose()
            return False
        else:
            return True

    def set_sink(self, sink, xid):
        self.imagesink = sink
        self.imagesink.set_window_handle(xid)



# Copyright (c) 2011-13 Walter Bender
# Copyright (c) 2011 Collabora Ltd. <http://www.collabora.co.uk/>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import os

from gettext import gettext as _

from gi.repository import GdkPixbuf

from TurtleArt.tautils import data_to_string, data_from_string, get_path, \
    base64_to_image, debug_output, error_output
from TurtleArt.taconstants import DEFAULT_TURTLE_COLORS

from sugar3 import profile
from sugar3.presence import presenceservice

from .textchannelwrapper import CollabWrapper


SERVICE = 'org.laptop.TurtleArtActivity'
IFACE = SERVICE
PATH = '/org/laptop/TurtleArtActivity'


class Collaboration():

    def __init__(self, tw, activity):
        """ A simplistic sharing model: the sharer is the master """
        self._tw = tw
        self._tw.send_event = self.send_event
        self._tw.remote_turtle_dictionary = {}
        self._activity = activity
        self._setup_dispatch_table()

    def setup(self):
        # TODO: hand off role of master if sharer leaves
        self.pservice = presenceservice.get_instance()
        self.initiating = None  # sharing (True) or joining (False)

        # Add my buddy object to the list
        owner = self.pservice.get_owner()
        self.owner = owner
        self._tw.buddies.append(self.owner)
        self._share = ''
        self._activity.connect('shared', self._activity_shared_cb)
        self._activity.connect('joined', self._activity_joined_cb)

        self.collab = CollabWrapper(self._activity)
        self.collab.connect('message', self._message_cb)
        self.collab.connect('joined', self._joined_cb)
        self.collab.setup()

    def _setup_dispatch_table(self):
        self._processing_methods = {
            't': self._turtle_request,
            'T': self._receive_turtle_dict,
            'R': self._reskin_turtle,
            'f': self._move_forward,
            'a': self._move_in_arc,
            'r': self._rotate_turtle,
            'x': self._set_xy,
            'W': self._draw_text,
            'c': self._set_pen_color,
            'g': self._set_pen_gray_level,
            's': self._set_pen_shade,
            'w': self._set_pen_width,
            'p': self._set_pen_state,
            'F': self._fill_polygon,
            'P': self._draw_pixbuf,
            'B': self._paste,
            'S': self._speak
        }

    def _activity_shared_cb(self, activity):
        self._tw.set_sharing(True)

        self.initiating = True
        self.waiting_for_turtles = False
        self._tw.remote_turtle_dictionary = self._get_dictionary()
        self._enable_share_button()

    def _activity_joined_cb(self, activity):
        self._tw.set_sharing(True)

        self.initiating = False
        # Joiner is to request current state from sharer.
        self.waiting_for_turtles = True
        self._enable_share_button()

    def _joined_cb(self, collab):
        # Now that we have the tube, we can ask for the turtle dictionary.
        debug_output('Sending a request for the turtle dictionary',
                     self._tw.running_sugar)
        # We need to send our own nick, colors, and turtle position
        colors = self._get_colors()
        event = data_to_string([self._get_nick(), colors])
        debug_output(event, self._tw.running_sugar)
        self.send_event('t', event)

    def _enable_share_button(self):
        self._activity.share_button.set_icon_name('shareon')
        self._activity.share_button.set_tooltip(_('Share selected blocks'))

    def _message_cb(self, colab, buddy, msg):
        """
        Events are sent as action and payload.  Everyone gets the
        turtle dictionary from the sharer and watches for 't' events,
        which indicate that a new turtle has joined.
        """

        action = msg.get('action')

        if action in self._processing_methods:
            save_active_turtle = self._tw.turtles.get_active_turtle()
            self._processing_methods[action](msg.get('event'))
            self._tw.turtles.set_turtle(
                self._tw.turtles.get_turtle_key(save_active_turtle))
            return

        if action == '!!ACTION_INIT_REQUEST':
            return

        error_output('unhandled action %r' % action)

    def send_event(self, action, event):
        """ Send event through the tube. """
        event = {'action': action, 'event': event}

        self.collab.post(event)

    def _turtle_request(self, payload):
        ''' incoming turtle from a joiner '''
        if payload > 0:
            [nick, colors] = data_from_string(payload)
            # FIXME: nick may not be unique, use buddy hash
            if nick != self._tw.nick:  # It is not me.
                # There may not be a turtle dictionary.
                if hasattr(self._tw, 'remote_turtle_dictionary'):
                    # Make sure it is not a "rejoin".
                    if nick not in self._tw.remote_turtle_dictionary:
                        # Add new turtle for the joiner.
                        self._tw.turtles.set_turtle(nick, colors)
                        self._tw.label_remote_turtle(nick, colors)
                    self._tw.remote_turtle_dictionary[nick] = colors
                else:
                    self._tw.remote_turtle_dictionary = self._get_dictionary()
                    # Add new turtle for the joiner.
                    self._tw.turtles.set_turtle(nick, colors)
                    self._tw.label_remote_turtle(nick, colors)

        # Sharer should send the updated remote turtle dictionary to everyone.
        if self.initiating:
            if self._tw.nick not in self._tw.remote_turtle_dictionary:
                self._tw.remote_turtle_dictionary[self._tw.nick] = \
                    self._get_colors()
            event = data_to_string(self._tw.remote_turtle_dictionary)
            self.send_event('T', event)
            self.send_my_xy()  # And the sender should report her xy position.

    def _receive_turtle_dict(self, payload):
        ''' Any time there is a new joiner, an updated turtle dictionary is
        circulated. Everyone must report their turtle positions so that we
        are in sync. '''
        if self.waiting_for_turtles:
            if len(payload) > 0:
                # Grab the new remote turtles dictionary.
                remote_turtle_dictionary = data_from_string(payload)
                # Add see what is new.
                for nick in remote_turtle_dictionary:
                    if nick == self._tw.nick:
                        debug_output('skipping my nick %s' %
                                     (nick), self._tw.running_sugar)
                    elif nick != self._tw.remote_turtle_dictionary:
                        # Add new the turtle.
                        colors = remote_turtle_dictionary[nick]
                        self._tw.remote_turtle_dictionary[nick] = colors
                        self._tw.turtles.set_turtle(nick, colors)
                        # Label the remote turtle.
                        self._tw.label_remote_turtle(nick, colors)
                        debug_output('adding %s to remote turtle dictionary' %
                                     (nick), self._tw.running_sugar)
                    else:
                        debug_output('%s already in remote turtle dictionary' %
                                     (nick), self._tw.running_sugar)
            self.waiting_for_turtles = False
        self.send_my_xy()

    def send_my_xy(self):
        ''' Set xy location so joiner can sync turtle positions. Should be
        used to sync positions after turtle drag. '''
        self._tw.turtles.set_turtle(self._get_nick())
        if self._tw.turtles.get_active_turtle().get_pen_state():
            self.send_event('p', data_to_string([self._get_nick(), False]))
            put_pen_back_down = True
        else:
            put_pen_back_down = False
        self.send_event(
            'x', data_to_string(
                [self._get_nick(),
                 [int(self._tw.turtles.get_active_turtle().get_xy()[0]),
                  int(self._tw.turtles.get_active_turtle().get_xy()[1])]]))
        if put_pen_back_down:
            self.send_event('p', data_to_string([self._get_nick(), True]))
        self.send_event(
            'r', data_to_string(
                [self._get_nick(),
                 int(self._tw.turtles.get_active_turtle().get_heading())]))

    def _reskin_turtle(self, payload):
        if len(payload) > 0:
            [nick, [width, height, data]] = data_from_string(payload)
            if nick != self._tw.nick:
                if self._tw.running_sugar:
                    tmp_path = get_path(self._tw.activity, 'instance')
                else:
                    tmp_path = '/tmp'
                file_name = base64_to_image(data, tmp_path)
                pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(file_name,
                                                                width, height)
                self._tw.turtles.set_turtle(nick)
                self._tw.turtles.get_active_turtle().set_shapes([pixbuf])

    def _draw_pixbuf(self, payload):
        if len(payload) > 0:
            [nick, [a, b, x, y, w, h, width, height, data]] =\
                data_from_string(payload)
            if nick != self._tw.nick:
                if self._tw.running_sugar:
                    tmp_path = get_path(self._tw.activity, 'instance')
                else:
                    tmp_path = '/tmp'
                file_name = base64_to_image(data, tmp_path)
                pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(file_name,
                                                                width, height)
                pos = self._tw.turtles.turtle_to_screen_coordinates((x, y))
                self._tw.turtles.get_active_turtle().draw_pixbuf(
                    pixbuf, a, b, pos[0], pos[1], w, h, file_name, False)

    def _move_forward(self, payload):
        if len(payload) > 0:
            [nick, x] = data_from_string(payload)
            if nick != self._tw.nick:
                self._tw.turtles.set_turtle(nick)
                self._tw.turtles.get_active_turtle().forward(x, False)

    def _move_in_arc(self, payload):
        if len(payload) > 0:
            [nick, [a, r]] = data_from_string(payload)
            if nick != self._tw.nick:
                self._tw.turtles.set_turtle(nick)
                self._tw.turtles.get_active_turtle().arc(a, r, False)

    def _rotate_turtle(self, payload):
        if len(payload) > 0:
            [nick, h] = data_from_string(payload)
            if nick != self._tw.nick:
                self._tw.turtles.set_turtle(nick)
                self._tw.turtles.get_active_turtle().set_heading(h, False)

    def _set_xy(self, payload):
        if len(payload) > 0:
            [nick, [x, y]] = data_from_string(payload)
            if nick != self._tw.nick:
                self._tw.turtles.set_turtle(nick)
                self._tw.turtles.get_active_turtle().set_xy(x, y, share=False)

    def _draw_text(self, payload):
        if len(payload) > 0:
            [nick, [label, x, y, size, w]] = data_from_string(payload)
            if nick != self._tw.nick:
                self._tw.turtles.set_turtle(nick)
                self._tw.turtles.get_active_turtle().draw_text(
                    label, x, y, size, w, False)

    def _set_pen_color(self, payload):
        if len(payload) > 0:
            [nick, x] = data_from_string(payload)
            if nick != self._tw.nick:
                self._tw.turtles.set_turtle(nick)
                self._tw.turtles.get_active_turtle().set_color(x, False)

    def _set_pen_gray_level(self, payload):
        if len(payload) > 0:
            [nick, x] = data_from_string(payload)
            if nick != self._tw.nick:
                self._tw.turtles.set_turtle(nick)
                self._tw.turtles.get_active_turtle().set_gray(x, False)

    def _set_pen_shade(self, payload):
        if len(payload) > 0:
            [nick, x] = data_from_string(payload)
            if nick != self._tw.nick:
                self._tw.turtles.set_turtle(nick)
                self._tw.turtles.get_active_turtle().set_shade(x, False)

    def _set_pen_width(self, payload):
        if len(payload) > 0:
            [nick, x] = data_from_string(payload)
            if nick != self._tw.nick:
                self._tw.turtles.set_turtle(nick)
                self._tw.turtles.get_active_turtle().set_pen_size(x, False)

    def _set_pen_state(self, payload):
        if len(payload) > 0:
            [nick, x] = data_from_string(payload)
            if nick != self._tw.nick:
                self._tw.turtles.set_turtle(nick)
                self._tw.turtles.get_active_turtle().set_pen_state(x, False)

    def _fill_polygon(self, payload):
        if len(payload) > 0:
            [nick, poly_points] = data_from_string(payload)
            shared_poly_points = []
            for i in range(len(poly_points)):
                x, y = self._tw.turtles.screen_to_turtle_coordinates(
                    (poly_points[i][1], poly_points[i][2]))
                if poly_points[i][0] in ['move', 'line']:
                    shared_poly_points.append((poly_points[i][0], x, y))
                elif poly_points[i][0] in ['rarc', 'larc']:
                    shared_poly_points.append(
                        (poly_points[i][0],
                         x,
                         y,
                         poly_points[i][3],
                            poly_points[i][4],
                            poly_points[i][5]))
            if nick != self._tw.nick:
                self._tw.turtles.set_turtle(nick)
                self._tw.turtles.get_active_turtle().set_poly_points(
                    shared_poly_points)
                self._tw.turtles.get_active_turtle().stop_fill(False)

    def _speak(self, payload):
        if len(payload) > 0:
            [nick, language_option, text] = data_from_string(payload)
            if language_option == 'None':
                language_option = ''
            if text is not None:
                if self._tw.running_sugar:
                    from sugar3.speech import SpeechManager
                    sm = SpeechManager()
                    sm.say_text(text)
                else:
                    os.system(
                        'espeak %s "%s" --stdout | aplay' %
                        (language_option, str(text)))

    def _paste(self, payload):
        if len(payload) > 0:
            [nick, text] = data_from_string(payload)
            if text is not None:
                self._tw.process_data(data_from_string(text),
                                      self._tw.paste_offset)
                self._tw.paste_offset += 20

    def _get_dictionary(self):
        return {self._get_nick(): self._get_colors()}

    def _get_nick(self):
        return self._tw.nick

    def _get_colors(self):
        colors = None
        if self._tw.running_sugar:
            if profile.get_color() is not None:
                colors = profile.get_color().to_string()
        else:
            colors = self._activity.get_colors()
        if colors is None:
            colors = '%s,%s' % (DEFAULT_TURTLE_COLORS[0],
                                DEFAULT_TURTLE_COLORS[1])
        return colors.split(',')



# Copyright (c) 2010-11, Tony Forster

# This procedure is invoked when the user-definable block on the
# "extras" palette is selected.

# Usage: Import this code into a Python (user-definable) block and
# pass a frequency in Hertz to the Python block. A tone will play over
# the speaker at the specified frequency.


def myblock(tw, args):
    ''' Plays a sound at frequency frequency '''

    import os
    frequency = args[0]
    os.system('speaker-test -t sine -l 1 -f %d' % (int(frequency)))



# Copyright (c) 2008, Media Modifications Ltd.
# Copyright (c) 2011, Walter Bender

# This procedure is invoked when the user-definable block on the
# "extras" palette is selected.

# Usage: Import this code into a Python (user-definable) block; Pass
# it 'start' to start recording; 'stop' to stop recording; 'play' to
# play back your recording; or 'save' to save your recording to the
# Sugar Journal.


def myblock(tw, args):
    ''' Record and playback a sound (Sugar only) '''
    import os
    import gst
    import gobject
    gobject.threads_init()

    from TurtleArt.tautils import get_path
    from TurtleArt.tagplay import play_audio_from_file
    from sugar.datastore import datastore
    from sugar import profile

    from gettext import gettext as _

    class Grecord:

        ''' A class for creating a gstreamer session for recording audio. '''

        def __init__(self, tw):
            ''' Set up the stream. We save to a raw .wav file and then
            convert the sound to .ogg for saving. '''
            datapath = get_path(tw.parent, 'instance')
            self.capture_file = os.path.join(datapath, 'output.wav')
            self.save_file = os.path.join(datapath, 'output.ogg')
            self._eos_cb = None

            self._can_limit_framerate = False
            self._recording = False

            self._audio_transcode_handler = None
            self._transcode_id = None

            self._pipeline = gst.Pipeline("Record")
            self._create_audiobin()
            self._pipeline.add(self._audiobin)

            bus = self._pipeline.get_bus()
            bus.add_signal_watch()
            bus.connect('message', self._bus_message_handler)

        def _create_audiobin(self):
            ''' Assemble all the pieces we need. '''
            src = gst.element_factory_make("alsasrc", "absrc")

            # attempt to use direct access to the 0,0 device, solving
            # some A/V sync issues
            src.set_property("device", "plughw:0,0")
            hwdev_available = src.set_state(gst.STATE_PAUSED) != \
                gst.STATE_CHANGE_FAILURE
            src.set_state(gst.STATE_NULL)
            if not hwdev_available:
                src.set_property("device", "default")

            srccaps = gst.Caps(
                "audio/x-raw-int,rate=16000,channels=1,depth=16")

            # guarantee perfect stream, important for A/V sync
            rate = gst.element_factory_make("audiorate")

            # without a buffer here, gstreamer struggles at the start
            # of the recording and then the A/V sync is bad for the
            # whole video (possibly a gstreamer/ALSA bug -- even if it
            # gets caught up, it should be able to resync without
            # problem)
            queue = gst.element_factory_make("queue", "audioqueue")
            queue.set_property("leaky", True)  # prefer fresh data
            queue.set_property("max-size-time", 5000000000)  # 5 seconds
            queue.set_property("max-size-buffers", 500)

            enc = gst.element_factory_make("wavenc", "abenc")

            sink = gst.element_factory_make("filesink", "absink")
            sink.set_property("location", self.capture_file)

            self._audiobin = gst.Bin("audiobin")
            self._audiobin.add(src, rate, queue, enc, sink)

            src.link(rate, srccaps)
            gst.element_link_many(rate, queue, enc, sink)

        def is_recording(self):
            ''' Are we recording? '''
            return self._recording

        def _get_state(self):
            ''' What is the state of our gstreamer pipeline? '''
            return self._pipeline.get_state()[1]

        def start_recording_audio(self):
            ''' Start the stream in order to start recording. '''
            if self._get_state() == gst.STATE_PLAYING:
                return
            self._pipeline.set_state(gst.STATE_PLAYING)
            self._recording = True

        def stop_recording_audio(self):
            ''' Stop recording and then convert the results into a
            .ogg file using a new stream. '''
            self._pipeline.set_state(gst.STATE_NULL)
            self._recording = False

            if not os.path.exists(self.capture_file) or \
                    os.path.getsize(self.capture_file) <= 0:
                return

            # Remove previous transcoding results.
            if os.path.exists(self.save_file):
                os.remove(self.save_file)

            line = 'filesrc location=' + self.capture_file + \
                ' name=audioFilesrc ! wavparse name=audioWavparse \
! audioconvert name=audioAudioconvert ! vorbisenc name=audioVorbisenc \
! oggmux name=audioOggmux ! filesink name=audioFilesink'
            audioline = gst.parse_launch(line)

            # vorbis_enc = audioline.get_by_name('audioVorbisenc')

            audioFilesink = audioline.get_by_name('audioFilesink')
            audioFilesink.set_property("location", self.save_file)

            audioBus = audioline.get_bus()
            audioBus.add_signal_watch()
            self._audio_transcode_handler = audioBus.connect(
                'message', self._onMuxedAudioMessageCb, audioline)
            self._transcode_id = gobject.timeout_add(
                200, self._transcodeUpdateCb, audioline)
            audioline.set_state(gst.STATE_PLAYING)

        def _transcodeUpdateCb(self, pipe):
            ''' Where are we in the transcoding process? '''
            position, duration = self._query_position(pipe)
            if position != gst.CLOCK_TIME_NONE:
                value = position * 100.0 / duration
                value = value / 100.0
            return True

        def _query_position(self, pipe):
            ''' Where are we in the stream? '''
            try:
                position, format = pipe.query_position(gst.FORMAT_TIME)
            except BaseException:
                position = gst.CLOCK_TIME_NONE

            try:
                duration, format = pipe.query_duration(gst.FORMAT_TIME)
            except BaseException:
                duration = gst.CLOCK_TIME_NONE

            return (position, duration)

        def _onMuxedAudioMessageCb(self, bus, message, pipe):
            ''' Clean up at end of stream.'''
            if message.type != gst.MESSAGE_EOS:
                return True

            gobject.source_remove(self._audio_transcode_handler)
            self._audio_transcode_handler = None
            gobject.source_remove(self._transcode_id)
            self._transcode_id = None
            pipe.set_state(gst.STATE_NULL)
            pipe.get_bus().remove_signal_watch()
            pipe.get_bus().disable_sync_message_emission()

            os.remove(self.capture_file)
            return False

        def _bus_message_handler(self, bus, message):
            ''' Handle any messages associated with the stream. '''
            t = message.type
            if t == gst.MESSAGE_EOS:
                if self._eos_cb:
                    cb = self._eos_cb
                    self._eos_cb = None
                    cb()
            elif t == gst.MESSAGE_ERROR:
                # TODO: if we come out of suspend/resume with errors, then
                # get us back up and running...  TODO: handle "No space
                # left on the resource.gstfilesink.c" err, debug =
                # message.parse_error()
                pass

    # We store the audio-record stream instance as tw.grecord so that
    # we can use it repeatedly.
    if not hasattr(tw, 'grecord'):
        tw.grecord = Grecord(tw)

    # Sometime we need to parse multiple arguments, e.g., save, savename
    save_name = '%s_%s' % (tw.activity.name, _('sound'))
    cmd = args[0].lower()
    if len(args) > 1:
        save_name = str(args[1])

    if cmd == 'start' or cmd == _('start').lower():
        tw.grecord.start_recording_audio()
    elif cmd == 'stop' or cmd == _('stop').lower():
        tw.grecord.stop_recording_audio()
    elif cmd == 'play' or cmd == _('play').lower():
        play_audio_from_file(tw.lc, tw.grecord.save_file)
    elif cmd == 'save' or cmd == _('save').lower():
        if os.path.exists(tw.grecord.save_file) and tw.running_sugar:
            dsobject = datastore.create()
            dsobject.metadata['title'] = save_name
            dsobject.metadata['icon-color'] = profile.get_color().to_string()
            dsobject.metadata['mime_type'] = 'audio/ogg'
            dsobject.set_file_path(tw.grecord.save_file)
            datastore.write(dsobject)
            dsobject.destroy()
